/*
 *  spica_struct_1A.c
 *  
 *	This program reads the contents of a SPICAM or SPICAV 1A file
 *	and returns a structure containing:
 *
 *                   - "Cleandata" field: Data cleaned from :
 *                     - Missing packets (set to NaN value)
 *                     - Erroneous data (only for SPICAM data, (set to NaN value))
 *                     - Saturated pixels (saturation value is 4095 ADU, (set to NaN value))
 *                     - Impacted by comisc ray pixels ((set to NaN value))
 *                     - Electronic noise
 *                     - Dark charge
 *                     format is a 3 dimensions float array containing the formatted data. 
 *                     For a general case 5-band observation, the dimensions of
 *                     the array are (Nb_Pixel=408,Nb_Spectrum,Nb_Band=5). For an Alignment
 *                     mode observation the dimensions are (Nb_PixelX=408,Nb_PixelY=289,
 *                     Nb_Image). Window mode dimensions are (XWidth,YWidth,Nb_Record)
 *
 *                  - "Flags" field: a 3 dimensions integer array of the same dimension as 
 *                    the data fields.
 *                    Flags have the following meaning:
 *                    0: OK
 *                    1: Missing data
 *                    2: Erroneous data (SPICAM)
 *                    3: Saturated data
 *                    4: Data damaged by comisc rays
 *                    5: Data corrected from electronic noise
 *                    
 *                  - "ErrorData" field:
 *                   
 *                  - "Info" sub-structure containing information about the
 *                    input files and general observation characteristics 
 *                    (tag names in parenthesis are specific to alignment
 *                    observations):
 *                       o NAxis1[Int]         : Number of pixels of the X-axis of the 
 *                                               CCD, i.e. number of points of each spec-
 *                                               trum = 408 except for window mode.
 *                       o NAxis2[Int]         : Number of 5-band spectra recorded throug-
 *                                               hout observation. For alignment observa-
 *                                               tion this field corresponds to the number
 *                                               of pixels of the Y-axis of the CCD = 289.
 *                                               For Window modes, it is the vertical width.
 *                       o NAxis3[Int]         : Number of bands of the CCD = 5. For align-
 *                                               ment observations this is equal to the
 *                                               number of complete mapping of the CCD.
 *                                               For Window mode this is equal to the number 
 *                                               of records.
 *                       o Instrument[String]  : Instrument name.
 *                       o Orbit[Int]          : Orbit index of the mission.
 *                       o Sequence[Int]       : Sequence index of the orbit.
 *                       o ObsType[Char]       : Type of observation. Cf SPICAM/V documen
 *                                               -tation.
 *                       o BeginTime[String]   : Observation first recording time, 
 *                                               format=YYYY-MM-DDTHH:MM:SS.mSmSmS.
 *                       o EndTime[String]     : Observation last recording time, 
 *                                               format=YYYY-MM-DDTHH:MM:SS.mSmSmS
 *                       o DCProc [Boolean]    : Dark Charge treatment
 *                       o ENProc [Boolean]    : Electronic Noise Treatment
 *                       o COSProc [Boolean]   : Cosmic rays pixels Treatment
 *                       o SATProc [Boolean]   : Saturated pixels Treatment
 *                       o File0C [String]     : 0C file used to build this 1A level
 *                       o OrbDCNU [Int]       : Number of the DCNU Model orbit
 *                       o Data_status [String]: Status of the Data (F "Final" or "P" Preliminary)
 *                       o Geo_status [String] : Status of the Geometry (F "Final" or "P" Preliminary)
 *                       o Flag_status [String]: Status of the Flag (F "Final" or "P" Preliminary)
 *                       o DC_status [String]  : Status of the Dark Charge correction (F "Final" or "P" Preliminary)
 *                     
 *                 - "Parameters" sub-structure containing functional parameters 
 *                   of the instrument. For 5-bands observations the "All_Ti" and 
 *                   "T_..." fields are 1d of length the number of spectra. For 
 *                   alignment modes they are 2d with size [nb_PixelY=289,nb_Image]:
 *                       o CodeOp[Int]     : Operational code of observation. Cf SPICAM/V docu-
 *                                           mentation.
 *                       o Binning[Int]    : Number of lines binned together to form a band. In 
 *                                           case of progressive binning, the field si set to
 *                                           string 'VARIABLE'.
 *                       o HT[Int]         : Digital scale of the High Voltage used to amplify 
 *                                           signal. Ranging from 0 to 255. See SPICAM/V docu-
 *                                           mentation for corresponding voltage.
 *                       o Ti[Int]         : Exposure Time of the CCD, in second/100. Usually 
 *                                           constant throughout observation. If not the field is 
 *                                           set to string 'VARIABLE'.
 *                       o X0[Int]         : X-axis first pixel index.
 *                       o Y0[Int]         : Y-axis first pixel index. Field is set to 'VARIABLE'
 *                                           for alignment observations.
 *                       o Slit[boolean]   : 1 if slit is on, 0 else.
 *                       o Peltier[boolean]: 1 if Peltier cooler is on, 0 else.
 *                       o UVSampling[Int] : Sample rate of the spectrometer.
 *                       o IROn[boolean]   : 1 if IR channel is on, 0 else.
 *                       o SoirOn[boolean] : 1 if SPICAV SOIR channel is on, 0 else. 255 for 
 *                                           SPICAM.
 *                       o All_Ti[Int]     : vector of all the time exposure throughout obser-
 *                                           vation.
 *                       o T_Peltier[Int]  : Temperature vector of Peltier's hot side (all tem-
 *                                           peratures ar given in °C).
 *                       o T_CCD[Int]      : Temperature vector of the CCD.
 *                       o T_NumBoard[Int] : Temperature vector of the digital board.
 *                       o T_BTBoard[Int]  : Temperature vector of the low tension board.
 *                       o T_Shutter[Int]  : Temperature vector of the solar shutter.
 *                       o T_ServBoard[Int]: Temperature vector of the servitudes board.
 *                       o T_HVPS[Int]     : Temperature vector of the High Voltage Power
 *                                           Supply.
 *                       o T_Structure[Int]: Temperature vector of the structure.
 *                     
 *                 - "GeoInfo" sub-structure containing geometrical parameters 
 *                   of the instrument which do not vary during observation:
 *                       o Target[String]   : Pointing target of the instrument. Cf SPICAM/V 
 *                                            documentation for the star catalog.
 *                       o SunLat[Float]    : Sun latitude planet body-fixed frame, at 
 *                                            first record (deg).
 *                       o SunLong[Float]   : Sun longitude planet body-fixed frame, at
 *                                            first record (deg).
 *                       o SunDist[ULong]   : Distance between planet and sun, at first
 *                                            record (ua).
 *                       o SunLS[Float]     : Sun solar longitude in the planet body-fixed
 *                                            frame, at first record (deg).
 *                       o SunRa[Float]     : Sun Ra, at first record (deg).
 *                       o SunDec[Float]    : Sun Dec, at first record (deg).
 *                       o SlitCenter[Float]: Position vector of the slit center in SC axes 
 *                                            (=offset, deg).
 *                       o ShadowCone[Char] : Trajectory of the spacecraft relatively to the 
 *                                            planet shadow cone. 
 *
 *                 - "Geo" sub-structure containing the time, and geometrical
 *                   parameters which vary during observation. Parameters are
 *                   grouped by the following nested structure. For 5-bands observations
 *                   the array is 1d of length the number of spectra. For alignment modes
 *                   the array is 2d with size [nb_PixelY=289,nb_Image].
 *                   NOTA: Depending on the type and mode of observations, the 
 *                   sub-structures vary. Common sub-structures are: {Record,
 *                   Spacecraft,Band3,Coordinates,TransMatrix}. All 5-band obser-
 *                   vations have the folllowing added sub-structures: {Band1,
 *                   Band2,Band4,Band5}. Star occultations have another sub-struc-
 *                   ture which is {LineOfSight}. 
 *                   Alignment modes have the specific sub-structure {CCDLine}.
 *                       o Record: + Number[Int] : observation record number, beginning at 1.
 *                                 + Time[String]: Record time, format=YYYY-MM-DDTHH:MM:SS.mSmSmS
 *                    
 *                       o Spacecraft: + Lat[Float]            : Latitude of the SC (deg).
 *                                     + Long[Float]           : Longitude of the SC (deg).
 *                                     + Alt[Float]            : Altitude of the SC in (km).
 *                                     + SZA[Float]            : Solar zenith angle of the SC (deg).
 *                                     + ShadowConeRatio[Float]: Ratio giving position of SC towards 
 *                                                               planet shadow cone. If >1 then SC is
 *                                                               outside the shadow cone else it's inside.
 *
 *                       o Band3: + Sza[Float]                : Solar zenith angle of the 3rd band/central 
 *                                                              line of the CCD portion (deg).
 *                                + DistToPlanetNP[Float]     : Distance to the planet nearest point (km).
 *                                + PixelSize[Float]          : Pixel size at planet nearest point (km).
 *                                + AngleLOSSun[Float]        : Angle between line of sight and sun (deg).
 *                                + PhaseLOSSun[Float]        : Phase between line of sight at planet near-
 *                                                              est point and sun(deg).
 *                                + SolarIncidence[Float]     : Solar incidence between normal at planet 
 *                                                              nearest point and sun (deg)
 *                                + SolarLocalTime[Float]     : Solar local time at planet nearest point (deg).
 *                                + AngleNormalObserver[Float]: Angle between Normal and Observer (deg).
 *                                + DistLOSPlanetCenter[Float]: Distance between line of sight and planet 
 *                                                              center (km).
 *                      
 *                            For non alignment modes only:
 *                                + Lat[Float] : Latitude of the 3rd Band  (deg)             
 *                                + Long[Float]: Longitude of the 3rd Band (deg)              
 *                                + Alt[Float] : Altitude of the 3rd Band (km)               
 *                                + Ra[Float]  : Ra of the 3rd Band  (deg)   
 *                                + Dec[Float] : Dec of the 3rd Band (deg)
 *
 *                       o Band1: + Lat[Float] : Latitude of the 1st Band (deg)              
 *                                + Long[Float]: Longitude of the 1st Band (deg)              
 *                                + Alt[Float] : Altitude of the 1st Band (km)               
 *                                + Ra[Float]  : Ra of the 1st Band (deg)     
 *                                + Dec[Float] : Dec of the 1st Band (deg)
 *
 *                       o Band2,Band4 and Band5 have the same corresponding fields.
 *
 *                       o CCDLine has the same corresponding fields for each line 
 *                         of the CCD. For alignment observations this structure replaces
 *                         the missing Band3 fields.
 *
 *                       o LOSE: Same fields as Band3 common fields, but associated to 
 *                               the line of sight emanating from the UV CCD center, but 
 *                               defined by S/C position and Star direction.
 * 
 *                       o Coordinates: + Xsc_X[Float]: X coordinate of the SC X-axis in MARSIAU/IAU_VENUS.
 *                                      + Xsc_Y[Float]: Y coordinate of the SC X-axis in MARSIAU/IAU_VENUS.
 *                                      + Xsc_Z[Float]: Z coordinate of the SC X-axis in MARSIAU/IAU_VENUS.
 *                                      + Ysc_X[Float]: X coordinate of the SC Y-axis in MARSIAU/IAU_VENUS.
 *                                      + Ysc_Y[Float]: Y coordinate of the SC Y-axis in MARSIAU/IAU_VENUS.
 *                                      + Ysc_Z[Float]: Z coordinate of the SC Y-axis in MARSIAU/IAU_VENUS.
 *                                      + Zsc_X[Float]: X coordinate of the SC Z-axis in MARSIAU/IAU_VENUS.
 *                                      + Zsc_Y[Float]: Y coordinate of the SC Z-axis in MARSIAU/IAU_VENUS.
 *                                      + Zsc_Z[Float]: Z coordinate of the SC Z-axis in MARSIAU/IAU_VENUS.
 *                                      + Angle_Xsc_LocalVert[Float]: Angle between Xsc and local verti-
 *                                                                    cal at planet nearest point (deg).
 *                                      + Angle_Xsc_Proj[Float]     : Angle between Xsc and the projec-
 *                                                                    tion plan (PP) U-axis (deg).
 *                                      + P1_u[Float]: U coordinate of the horizon point in the PP.
 *                                      + P1_v[Float]: V coordinate of the horizon point in the PP.
 *                                      + P2_u[Float]: U coordinate of the horizon point in the PP.
 *                                      + P2_v[Float]: V coordinate of the horizon point in the PP.
 *                                      + P3_u[Float]: U coordinate of the horizon point in the PP.
 *                                      + P3_v[Float]: V coordinate of the horizon point in the PP.
 *                                      + P4_u[Float]: U coordinate of the horizon point in the PP.
 *                                      + P4_v[Float]: V coordinate of the horizon point in the PP.
 *                                      + P5_u[Float]: U coordinate of the horizon point in the PP.
 *                                      + P5_v[Float]: V coordinate of the horizon point in the PP.
 *
 *                       o TransMatrix: + X_Ra [Float]: Ra of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 *                                                      formation matrix X vector (deg).
 *                                      + X_Dec[Float]: Dec of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 *                                                      formation matrix X vector (deg).
 *                                      + Y_Ra [Float]: Ra of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 *                                                      formation matrix Y vector (deg).
 *                                      + Y_Dec[Float]: Dec of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 *                                                      formation matrix Y vector (deg).
 *                                      + Z_Ra [Float]: Ra of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 *                                                      formation matrix Z vector (deg).
 *                                      + Z_Dec[Float]: Dec of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 *                                                      formation matrix Z vector (deg).
 *
 *  SVN Revision:
 *      $Rev: 807 $
 *      $Author: chapron $
 *      $Date: 2012-09-25 15:18:26 +0200 (mar., 25 sept. 2012) $
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "fitsio.h"
#include "exit_if.h"
#include "spica_struct_1A.h"

void spica_struct_1A(spica_1Afile *output,char *spica_1Afname, int mode)
{
	
	fitsfile *fptr;
	int status=0;
	
	if(mode != 0)
		printf("Opening file %s ... \n",spica_1Afname);
	
	fits_open_file(&fptr, spica_1Afname, READONLY, &status);
	EXIT_IF(status==104,"could not open the named file. Check for existence and correct path.");
	
	if(mode != 0)
		printf("Reading Info... \n");
    /*****************  LECTURE INFO  ********************/
	read_Info(fptr,output,status);
	
	if (status)
		fits_report_error(stderr, status);
    /*---------------------------------------------------*/
	
	if(mode != 0)
		printf("Reading Data... \n");
	/**************** LECTURE DONNEES  *******************/
	read_Data(fptr,output,status);
	
	if (status)
		fits_report_error(stderr, status);
    /*---------------------------------------------------*/
	
	if(mode != 0)
		printf("Reading Error Data... \n");
	/**************** LECTURE ErrData  *******************/
	read_ErrData(fptr,output,status);
	
	if (status)
		fits_report_error(stderr, status);
    /*---------------------------------------------------*/
	
	if(mode != 0)
		printf("Reading Flag... \n");
	/**************** LECTURE FLAG  *******************/
	read_Flag(fptr,output,status);
	
	if (status)
		fits_report_error(stderr, status);
    /*---------------------------------------------------*/
		
	if(mode != 0)
		printf("Reading Parameters... \n");
	/****************** LECTURE PARAM  ********************/
	read_Parameters(fptr,output, status);
	
	if (status)
		fits_report_error(stderr, status);
    /*---------------------------------------------------*/
		
	if(mode != 0)
		printf("Reading GeoInfo... \n");
	//***************** LECTURE GEOINFO ******************/
	read_GeoInfo(fptr,output, status);
	
	if (status)
		fits_report_error(stderr, status);
    /*---------------------------------------------------*/
	
	if(mode != 0)
		printf("Reading Geo... \n");
	//******************* LECTURE GEO ********************/
	read_Geo(fptr,output,status);
	
	if (status)
		fits_report_error(stderr, status);
    /*---------------------------------------------------*/
	
	//Fermeture fichier
	fits_close_file(fptr, &status);
	if(mode != 0)
		printf("End of reading file \n\n");
	
}
/****************************************************************************/


/****************************************************************************/
void read_Info(fitsfile *fptr, spica_1Afile *output,int status){
	
	int nfound;
	long naxes[3];
	
	fits_read_keys_lng(fptr, "NAXIS", 1, 3, naxes, &nfound, &status);
	
	output->Info.naxis1 = naxes[0];
	output->Info.naxis2 = naxes[1];
	output->Info.naxis3 = naxes[2];
	
	output->Info.instrument = malloc(FLEN_VALUE);
	EXIT_IF(!output->Info.instrument,"Error: memory allocation failure");
	output->Info.obstype = malloc(FLEN_VALUE);
	EXIT_IF(!output->Info.obstype,"Error: memory allocation failure");
	output->Info.begins = malloc(FLEN_VALUE);
	EXIT_IF(!output->Info.begins,"Error: memory allocation failure");
	output->Info.ends = malloc(FLEN_VALUE);
	EXIT_IF(!output->Info.ends,"Error: memory allocation failure");
	output->Info.File0C = malloc(FLEN_VALUE);
	EXIT_IF(!output->Info.File0C,"Error: memory allocation failure");
	output->Info.data_status = malloc(FLEN_VALUE);
	EXIT_IF(!output->Info.data_status,"Error: memory allocation failure");
	output->Info.geo_status = malloc(FLEN_VALUE);
	EXIT_IF(!output->Info.geo_status,"Error: memory allocation failure");
	output->Info.flag_status = malloc(FLEN_VALUE);
	EXIT_IF(!output->Info.flag_status,"Error: memory allocation failure");
	output->Info.dc_status = malloc(FLEN_VALUE);
	EXIT_IF(!output->Info.dc_status,"Error: memory allocation failure");

	fits_read_keyword(fptr,"INSTRU",output->Info.instrument,NULL,&status);
	fits_read_key(fptr,TINT,"ORBIT",&output->Info.orbit,NULL,&status);
	fits_read_key(fptr,TINT,"SEQ_NB",&output->Info.seq,NULL,&status);
	fits_read_keyword(fptr,"OBSTYPE",output->Info.obstype,NULL,&status);
	fits_read_keyword(fptr,"BEGINS",output->Info.begins,NULL,&status);
	fits_read_keyword(fptr,"ENDS",output->Info.ends,NULL,&status);
	fits_read_key(fptr,TINT,"DCProc",&output->Info.DCProc,NULL,&status);
	fits_read_key(fptr,TINT,"ENProc",&output->Info.ENProc,NULL,&status);
	fits_read_key(fptr,TINT,"COSProc",&output->Info.COSProc,NULL,&status);
	fits_read_key(fptr,TINT,"SATProc",&output->Info.SATProc,NULL,&status);
	fits_read_key(fptr,TINT,"ORBDCNU",&output->Info.orbDCNU,NULL,&status);
	fits_read_keyword(fptr,"DATA_0C",output->Info.File0C,NULL,&status);
	fits_read_keyword(fptr,"DATA_SS",output->Info.data_status,NULL,&status);
	fits_read_keyword(fptr,"GEO_SS",output->Info.geo_status,NULL,&status);
	fits_read_keyword(fptr,"FLAG_SS",output->Info.flag_status,NULL,&status);
	fits_read_keyword(fptr,"DC_SS",output->Info.dc_status,NULL,&status);
	
	output->Info.instrument = trimchars(output->Info.instrument);
	output->Info.obstype = trimchars(output->Info.obstype);
	output->Info.begins = trimchars(output->Info.begins);
	output->Info.ends = trimchars(output->Info.ends);
	output->Info.File0C = trimchars(output->Info.ends);
	output->Info.data_status = trimchars(output->Info.data_status);
	output->Info.geo_status = trimchars(output->Info.geo_status);
	output->Info.flag_status = trimchars(output->Info.flag_status);
	output->Info.dc_status = trimchars(output->Info.dc_status);
	
}
/****************************************************************************/


/****************************************************************************/
void read_Data(fitsfile *fptr, spica_1Afile *output,int status){
	
	int fpixel=1,ii,anynull;
	long npixelsperimg;
	float nullval=0;
	
	npixelsperimg = output->Info.naxis1 * output->Info.naxis2;
	
	for (ii=0; ii<output->Info.naxis3; ii++) {
		output->cleandata[ii] = malloc(npixelsperimg*sizeof(float));
		EXIT_IF(!output->cleandata[ii],"Error: memory allocation failure");
		fits_read_img(fptr, TFLOAT, fpixel+ii*npixelsperimg,npixelsperimg, &nullval,
					  output->cleandata[ii], &anynull, &status);
	}
	
}
/****************************************************************************/

/****************************************************************************/
void read_Flag(fitsfile *fptr, spica_1Afile *output,int status){
	
	int fpixel=1,ii,anynull,nullval=0;
	long npixelsperimg;
	
	fits_movnam_hdu(fptr,IMAGE_HDU,"Flag",0,&status);
	
	EXIT_IF(status,"Error: missing necessary Flag extension");
	
	npixelsperimg = output->Info.naxis1 * output->Info.naxis2;
	
	for (ii=0; ii<output->Info.naxis3; ii++) {
		output->flag[ii] = malloc(npixelsperimg*sizeof(int));
		EXIT_IF(!output->flag[ii],"Error: memory allocation failure");
		fits_read_img(fptr, TINT, fpixel+ii*npixelsperimg,npixelsperimg, &nullval,
					  output->flag[ii], &anynull, &status);
	}
	
}
/****************************************************************************/

/****************************************************************************/
void read_ErrData(fitsfile *fptr, spica_1Afile *output,int status){
	
	int fpixel=1,ii,anynull,nullval=0;
	long npixelsperimg;
	
	fits_movnam_hdu(fptr,IMAGE_HDU,"ErrData",0,&status);
	
	EXIT_IF(status,"Error: missing necessary ErrData extension");
	
	npixelsperimg = output->Info.naxis1 * output->Info.naxis2;
	
	for (ii=0; ii<output->Info.naxis3; ii++) {
		output->errdata[ii] = malloc(npixelsperimg*sizeof(int));
		EXIT_IF(!output->errdata[ii],"Error: memory allocation failure");
		fits_read_img(fptr, TFLOAT, fpixel+ii*npixelsperimg,npixelsperimg, &nullval,
					  output->errdata[ii], &anynull, &status);
	}
	
}
/****************************************************************************/


/****************************************************************************/
void read_Parameters(fitsfile *fptr, spica_1Afile *output,int status){
	
	int anynull;
	long naxis2,longnull;
	
	fits_movnam_hdu(fptr,BINARY_TBL,"Functional_Parameters",0,&status);
	EXIT_IF(status,"Error: missing necessary Functional_Parameters extension");
	
	fits_read_key(fptr,TLONG,"NAXIS2",&naxis2, NULL, &status);
	
	fits_read_key(fptr,TINT,"CODEOP",&output->Parameters.codeop,NULL,&status);
	fits_read_key(fptr,TINT,"BINNING",&output->Parameters.binning,NULL,&status);
	fits_read_key(fptr,TINT,"HT",&output->Parameters.ht,NULL,&status);
	fits_read_key(fptr,TINT,"TI",&output->Parameters.ti,NULL,&status);
	fits_read_key(fptr,TINT,"X0",&output->Parameters.x0,NULL,&status);
	fits_read_key(fptr,TINT,"Y0",&output->Parameters.y0,NULL,&status);
	fits_read_key(fptr,TINT,"SLIT",&output->Parameters.slit,NULL,&status);
	fits_read_key(fptr,TINT,"PELTIER",&output->Parameters.peltier,NULL,&status);
	fits_read_key(fptr,TINT,"UVSAMPL",&output->Parameters.uvsampling,NULL,&status);
	fits_read_key(fptr,TINT,"IR_ON",&output->Parameters.iron,NULL,&status);
	fits_read_key(fptr,TINT,"SOIR_ON",&output->Parameters.soiron,NULL,&status);
	
	output->Parameters.all_ti      = malloc(naxis2*sizeof(int));
	EXIT_IF(!output->Parameters.all_ti,"Error: memory allocation failure");
	output->Parameters.t_peltier   = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Parameters.t_peltier,"Error: memory allocation failure");
	output->Parameters.t_ccd       = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Parameters.t_ccd,"Error: memory allocation failure");
	output->Parameters.t_numboard  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Parameters.t_numboard,"Error: memory allocation failure");
	output->Parameters.t_btboard   = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Parameters.t_btboard,"Error: memory allocation failure");
	output->Parameters.t_shutter   = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Parameters.t_shutter,"Error: memory allocation failure");
	output->Parameters.t_servboard = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Parameters.t_servboard,"Error: memory allocation failure");
	output->Parameters.t_hvps      = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Parameters.t_hvps,"Error: memory allocation failure");
	output->Parameters.t_structure = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Parameters.t_structure,"Error: memory allocation failure");
	
	fits_read_col(fptr, TINT, 1, 1, 1, naxis2, &longnull, output->Parameters.all_ti,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Parameters.t_peltier,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Parameters.t_ccd,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Parameters.t_numboard,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Parameters.t_btboard,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Parameters.t_shutter,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Parameters.t_servboard,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 8, 1, 1, naxis2, &longnull, output->Parameters.t_hvps,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 9, 1, 1, naxis2, &longnull, output->Parameters.t_structure,
				  &anynull, &status);
	
}
/****************************************************************************/


/****************************************************************************/
void read_GeoInfo(fitsfile *fptr, spica_1Afile *output,int status){
	
	int ii=0;
	char *slit;
	char *delim = " ";
	char *token = NULL;
	
	
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Record",0,&status);
	EXIT_IF(status,"Error: missing necessary Geo_Record extension");
	
	slit = malloc(FLEN_VALUE);
	EXIT_IF(!slit,"Error: memory allocation failure");
	output->GeoInfo.target = malloc(FLEN_VALUE);	
	EXIT_IF(!output->GeoInfo.target,"Error: memory allocation failure");
	output->GeoInfo.shadowcone = malloc(FLEN_VALUE);
	EXIT_IF(!output->GeoInfo.shadowcone,"Error: memory allocation failure");
	
	fits_read_keyword(fptr,"TARGET",output->GeoInfo.target,NULL,&status);
	fits_read_key(fptr,TFLOAT,"SUNLAT",&output->GeoInfo.sunlat,NULL,&status);
	fits_read_key(fptr,TFLOAT,"SUNLONG",&output->GeoInfo.sunlong,NULL,&status);
	fits_read_key(fptr,TFLOAT,"SUNDIST",&output->GeoInfo.sundist,NULL,&status);
	fits_read_key(fptr,TFLOAT,"SUNLS",&output->GeoInfo.sunls,NULL,&status);
	fits_read_key(fptr,TFLOAT,"SUNRA",&output->GeoInfo.sunra,NULL,&status);
	fits_read_key(fptr,TFLOAT,"SUNDEC",&output->GeoInfo.sundec,NULL,&status);
	fits_read_keyword(fptr,"SLIT_C",slit,NULL,&status);
	fits_read_keyword(fptr,"CONE",output->GeoInfo.shadowcone,NULL,&status);
	
	slit = trimchars(slit);
	output->GeoInfo.target     = trimchars(output->GeoInfo.target);
	output->GeoInfo.shadowcone = trimchars(output->GeoInfo.shadowcone);
	
	for (token = strtok(slit, delim); token != NULL; token = strtok(NULL, delim))
	{
		char *unconverted;
		double value = strtod(token, &unconverted);
		if (!isspace(*unconverted) && *unconverted != 0)
		{
			printf("probleme conversion string->float\n");
		}
		output->GeoInfo.slitcenter[ii] = value;
		ii++;
	}
	
}
/****************************************************************************/



/****************************************************************************/
void read_Geo(fitsfile *fptr, spica_1Afile *output,int status){
	
	int ii=0,anynull;
	long naxis2,longnull;
	
	
	//Lecture Geo_record
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Record",0,&status);
	EXIT_IF(status,"Error: missing necessary Geo_Record extension");
	
	fits_read_key(fptr,TLONG,"NAXIS2",&naxis2, NULL, &status);
	
	output->Geo.Record.number = malloc(naxis2*sizeof(int));
	EXIT_IF(!output->Geo.Record.number,"Error: memory allocation failure");
	output->Geo.Record.time   = malloc(naxis2*sizeof(&output->Geo.Record.time));
	EXIT_IF(!output->Geo.Record.time,"Error: memory allocation failure");
	
	for (ii=0; ii<naxis2; ii++) {
		output->Geo.Record.time[ii] = malloc(24*sizeof(char));
		EXIT_IF(!output->Geo.Record.time[ii],"Error: memory allocation failure");
	}
	
	fits_read_col(fptr, TINT, 2, 1, 1, naxis2, &longnull, output->Geo.Record.number,
				  &anynull, &status);	
	fits_read_col(fptr, TSTRING, 1, 1, 1, naxis2, &longnull, output->Geo.Record.time,
				  &anynull, &status);
	
	
	
	//Lecture Geo_Spacecraft
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Spacecraft",0,&status);
	EXIT_IF(status,"Error: missing necessary Geo_Spacecraft extension");
	
	output->Geo.Spacecraft.lat = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Spacecraft.lat,"Error: memory allocation failure");
	output->Geo.Spacecraft.lon = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Spacecraft.lon,"Error: memory allocation failure");
	output->Geo.Spacecraft.alt = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Spacecraft.alt,"Error: memory allocation failure");
	output->Geo.Spacecraft.sza = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Spacecraft.sza,"Error: memory allocation failure");
	output->Geo.Spacecraft.shadowconeratio = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Spacecraft.shadowconeratio,"Error: memory allocation failure");
	
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.lat,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.lon,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.alt,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.sza,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Spacecraft.shadowconeratio,
				  &anynull, &status);
	
	
	
	
	//Lecture Geo_Band3
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band3",0,&status);
	EXIT_IF(status,"Error: missing necessary Geo_Band3 extension");
	
	output->Geo.Band3.sza				  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.sza,"Error: memory allocation failure");
	output->Geo.Band3.disttoplanetnp	  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.disttoplanetnp,"Error: memory allocation failure");
	output->Geo.Band3.pixelsize			  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.pixelsize,"Error: memory allocation failure");
	output->Geo.Band3.anglelossun		  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.anglelossun,"Error: memory allocation failure");
	output->Geo.Band3.phaselossun		  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.phaselossun,"Error: memory allocation failure");
	output->Geo.Band3.solarincidence	  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.solarincidence,"Error: memory allocation failure");
	output->Geo.Band3.solarlocaltime	  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.solarlocaltime,"Error: memory allocation failure");
	output->Geo.Band3.anglenormalobserver = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.anglenormalobserver,"Error: memory allocation failure");
	output->Geo.Band3.sunazimuth		  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.sunazimuth,"Error: memory allocation failure");
	output->Geo.Band3.distlosplanetcenter = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.distlosplanetcenter,"Error: memory allocation failure");
	output->Geo.Band3.distscplanetcenter  = malloc(naxis2*sizeof(float));
	EXIT_IF(!output->Geo.Band3.distscplanetcenter,"Error: memory allocation failure");
	
	output->Geo.Band3.lat   = calloc(naxis2,sizeof(float));
	EXIT_IF(!output->Geo.Band3.lat,"Error: memory allocation failure");
	output->Geo.Band3.lon = calloc(naxis2,sizeof(float));
	EXIT_IF(!output->Geo.Band3.lon,"Error: memory allocation failure");
	output->Geo.Band3.alt   = calloc(naxis2,sizeof(float));
	EXIT_IF(!output->Geo.Band3.alt,"Error: memory allocation failure");
	output->Geo.Band3.ra    = calloc(naxis2,sizeof(float));
	EXIT_IF(!output->Geo.Band3.ra,"Error: memory allocation failure");
	output->Geo.Band3.dec   = calloc(naxis2,sizeof(float));
	EXIT_IF(!output->Geo.Band3.dec,"Error: memory allocation failure");
	
	fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Geo.Band3.sza,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Geo.Band3.disttoplanetnp,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 8, 1, 1, naxis2, &longnull, output->Geo.Band3.pixelsize,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 9, 1, 1, naxis2, &longnull, output->Geo.Band3.anglelossun,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 10, 1, 1, naxis2, &longnull, output->Geo.Band3.phaselossun,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 11, 1, 1, naxis2, &longnull, output->Geo.Band3.solarincidence,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 12, 1, 1, naxis2, &longnull, output->Geo.Band3.solarlocaltime,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 13, 1, 1, naxis2, &longnull, output->Geo.Band3.anglenormalobserver,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 14, 1, 1, naxis2, &longnull, output->Geo.Band3.sunazimuth,
				  &anynull, &status);	
	fits_read_col(fptr, TFLOAT, 15, 1, 1, naxis2, &longnull, output->Geo.Band3.distlosplanetcenter,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 16, 1, 1, naxis2, &longnull, output->Geo.Band3.distscplanetcenter,
				  &anynull, &status);
	
	if (output->Parameters.codeop >= 100) {
		
		fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band3.lat,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band3.lon,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band3.alt,
					  &anynull, &status);	
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band3.ra,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band3.dec,
					  &anynull, &status);
	}
	
	
	//Lecture Geo_Band1,2,4 et 5 ou CCDLine
	
	output->Geo.Band1.lat   = calloc(naxis2,sizeof(float));
	output->Geo.Band1.lon = calloc(naxis2,sizeof(float));
	output->Geo.Band1.alt   = calloc(naxis2,sizeof(float));
	output->Geo.Band1.ra    = calloc(naxis2,sizeof(float));
	output->Geo.Band1.dec   = calloc(naxis2,sizeof(float));
	
	output->Geo.Band2.lat   = calloc(naxis2,sizeof(float));
	output->Geo.Band2.lon = calloc(naxis2,sizeof(float));
	output->Geo.Band2.alt   = calloc(naxis2,sizeof(float));
	output->Geo.Band2.ra    = calloc(naxis2,sizeof(float));
	output->Geo.Band2.dec   = calloc(naxis2,sizeof(float));	
	
	output->Geo.Band4.lat   = calloc(naxis2,sizeof(float));
	output->Geo.Band4.lon = calloc(naxis2,sizeof(float));
	output->Geo.Band4.alt   = calloc(naxis2,sizeof(float));
	output->Geo.Band4.ra    = calloc(naxis2,sizeof(float));
	output->Geo.Band4.dec   = calloc(naxis2,sizeof(float));
	
	output->Geo.Band5.lat   = calloc(naxis2,sizeof(float));
	output->Geo.Band5.lon = calloc(naxis2,sizeof(float));
	output->Geo.Band5.alt   = calloc(naxis2,sizeof(float));
	output->Geo.Band5.ra    = calloc(naxis2,sizeof(float));
	output->Geo.Band5.dec   = calloc(naxis2,sizeof(float));
	
	output->Geo.CCDLine.lat   = calloc(naxis2,sizeof(float));
	output->Geo.CCDLine.lon = calloc(naxis2,sizeof(float));
	output->Geo.CCDLine.alt   = calloc(naxis2,sizeof(float));
	output->Geo.CCDLine.ra    = calloc(naxis2,sizeof(float));
	output->Geo.CCDLine.dec   = calloc(naxis2,sizeof(float));
	
	if (output->Parameters.codeop >= 100) {
		
		fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band1",0,&status);
		EXIT_IF(status,"Error: missing necessary Geo_Band1 extension");
		
		fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band1.lat,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band1.lon,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band1.alt,
					  &anynull, &status);	
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band1.ra,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band1.dec,
					  &anynull, &status);
		
		
		fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band2",0,&status);
		EXIT_IF(status,"Error: missing necessary Geo_Band2 extension");
		
		fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band2.lat,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band2.lon,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band2.alt,
					  &anynull, &status);	
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band2.ra,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band2.dec,
					  &anynull, &status);
		
		
		fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band4",0,&status);
		EXIT_IF(status,"Error: missing necessary Geo_Band4 extension");
		
		fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band4.lat,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band4.lon,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band4.alt,
					  &anynull, &status);	
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band4.ra,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band4.dec,
					  &anynull, &status);
		
		
		fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Band5",0,&status);
		EXIT_IF(status,"Error: missing necessary Geo_Band5 extension");
		
		fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Band5.lat,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Band5.lon,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Band5.alt,
					  &anynull, &status);	
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Band5.ra,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Band5.dec,
					  &anynull, &status);
		
	}
	
	else {
		
		fits_movnam_hdu(fptr,BINARY_TBL,"Geo_CCDLine",0,&status);
		EXIT_IF(status,"Error: missing necessary Geo_CCDLine extension");
		
		fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.CCDLine.lat,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.CCDLine.lon,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.CCDLine.alt,
					  &anynull, &status);	
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.CCDLine.ra,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.CCDLine.dec,
					  &anynull, &status);
	}
	
	
	//Lecture Geo_LOSE
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_LOSE",0,&status);
	
	if (status == 0) {
		
		output->Geo.LOSE.lat = calloc(naxis2,sizeof(float));
		EXIT_IF(!output->Geo.LOSE.lat,"Error: memory allocation failure");
		output->Geo.LOSE.lon = calloc(naxis2,sizeof(float));
		EXIT_IF(!output->Geo.LOSE.lon,"Error: memory allocation failure");
		output->Geo.LOSE.alt = calloc(naxis2,sizeof(float));
		EXIT_IF(!output->Geo.LOSE.alt,"Error: memory allocation failure");
		output->Geo.LOSE.ra = calloc(naxis2,sizeof(float));
		EXIT_IF(!output->Geo.LOSE.ra,"Error: memory allocation failure");
		output->Geo.LOSE.dec = calloc(naxis2,sizeof(float));
		EXIT_IF(!output->Geo.LOSE.dec,"Error: memory allocation failure");
		output->Geo.LOSE.sza = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.sza,"Error: memory allocation failure");
		output->Geo.LOSE.disttoplanetnp = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.disttoplanetnp,"Error: memory allocation failure");
		output->Geo.LOSE.pixelsize = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.pixelsize,"Error: memory allocation failure");
		output->Geo.LOSE.anglelossun = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.anglelossun,"Error: memory allocation failure");
		output->Geo.LOSE.phaselossun = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.phaselossun,"Error: memory allocation failure");
		output->Geo.LOSE.solarincidence = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.solarincidence,"Error: memory allocation failure");
		output->Geo.LOSE.solarlocaltime = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.solarlocaltime,"Error: memory allocation failure");
		output->Geo.LOSE.anglenormalobserver = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.anglenormalobserver,"Error: memory allocation failure");
		output->Geo.LOSE.sunazimuth = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.sunazimuth,"Error: memory allocation failure");
		output->Geo.LOSE.distlosplanetcenter = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.distlosplanetcenter,"Error: memory allocation failure");
		output->Geo.LOSE.distscplanetcenter = malloc(naxis2*sizeof(float));
		EXIT_IF(!output->Geo.LOSE.distscplanetcenter,"Error: memory allocation failure");
		
		fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.LOSE.lat,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.LOSE.lon,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.LOSE.alt,
					  &anynull, &status);	
		fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.LOSE.ra,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.LOSE.dec,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Geo.LOSE.sza,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Geo.LOSE.disttoplanetnp,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 8, 1, 1, naxis2, &longnull, output->Geo.LOSE.pixelsize,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 9, 1, 1, naxis2, &longnull, output->Geo.LOSE.anglelossun,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 10, 1, 1, naxis2, &longnull, output->Geo.LOSE.phaselossun,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 11, 1, 1, naxis2, &longnull, output->Geo.LOSE.solarincidence,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 12, 1, 1, naxis2, &longnull, output->Geo.LOSE.solarlocaltime,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 13, 1, 1, naxis2, &longnull, output->Geo.LOSE.anglenormalobserver,
					  &anynull, &status);
		fits_read_col(fptr, TFLOAT, 14, 1, 1, naxis2, &longnull, output->Geo.LOSE.sunazimuth,
					  &anynull, &status);
		//fits_read_col(fptr, TFLOAT, 15, 1, 1, naxis2, &longnull, output->Geo.LOSE.distlosplanetcenter,
		//			  &anynull, &status);
		//fits_read_col(fptr, TFLOAT, 16, 1, 1, naxis2, &longnull, output->Geo.LOSE.distscplanetcenter,
		//			  &anynull, &status);
	}
	else status = 0;
	
	
	//Lecture Geo_Coordinates
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_Coordinates",0,&status);
	EXIT_IF(status,"Error: missing necessary Geo_Coordinates extension");
	
	output->Geo.Coordinates.xsc_x = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.xsc_y = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.xsc_z = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.ysc_x = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.ysc_y = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.ysc_z = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.zsc_x = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.zsc_y = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.zsc_z = malloc(naxis2*sizeof(float));
	
	output->Geo.Coordinates.angle_xsc_localvert = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.angle_xsc_proj = malloc(naxis2*sizeof(float));
	
	output->Geo.Coordinates.p1_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p1_v = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p2_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p2_v = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p3_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p3_v = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p4_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p4_v = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p5_u = malloc(naxis2*sizeof(float));
	output->Geo.Coordinates.p5_v = malloc(naxis2*sizeof(float));
	
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.Coordinates.xsc_x,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.Coordinates.xsc_y,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.Coordinates.xsc_z,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.Coordinates.ysc_x,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.Coordinates.ysc_y,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Geo.Coordinates.ysc_z,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 7, 1, 1, naxis2, &longnull, output->Geo.Coordinates.zsc_x,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 8, 1, 1, naxis2, &longnull, output->Geo.Coordinates.zsc_y,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 9, 1, 1, naxis2, &longnull, output->Geo.Coordinates.zsc_z,
				  &anynull, &status);
	
	fits_read_col(fptr, TFLOAT, 10, 1, 1, naxis2, &longnull, output->Geo.Coordinates.angle_xsc_localvert,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 11, 1, 1, naxis2, &longnull, output->Geo.Coordinates.angle_xsc_proj,
				  &anynull, &status);
	
	fits_read_col(fptr, TFLOAT, 12, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p1_u,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 13, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p1_v,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 14, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p2_u,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 15, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p2_v,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 16, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p3_u,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 17, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p3_v,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 18, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p4_u,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 19, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p4_v,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 20, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p5_u,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 21, 1, 1, naxis2, &longnull, output->Geo.Coordinates.p5_v,
				  &anynull, &status);
	
	//Lecture Geo_TransMatrix
	fits_movnam_hdu(fptr,BINARY_TBL,"Geo_TransMatrix",0,&status);
	EXIT_IF(status,"Error: missing necessary Geo_TransMatrix extension");
	
	output->Geo.TransMatrix.x_ra  = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.x_dec = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.y_ra  = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.y_dec = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.z_ra  = malloc(naxis2*sizeof(float));
	output->Geo.TransMatrix.z_dec = malloc(naxis2*sizeof(float));
	
	fits_read_col(fptr, TFLOAT, 1, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.x_ra,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 2, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.x_dec,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 3, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.y_ra,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 4, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.y_dec,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 5, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.z_ra,
				  &anynull, &status);
	fits_read_col(fptr, TFLOAT, 6, 1, 1, naxis2, &longnull, output->Geo.TransMatrix.z_dec,
				  &anynull, &status);
	
}
/****************************************************************************/



/****************************************************************************/
char *trimchars(char *str)
{
	char *end;
	
	// Trim begin '
	str++;
	
	// Trim leading space
	while(isspace(*str)) str++;
	
	if(*str == 0)  // All spaces?
		return str;
	
	// Trim end '
	end = str + strlen(str) - 1;
	end--;
	
	// Trim trailing space
	while(end > str && isspace(*end)) end--;
	
	// Write new null terminator
	*(end+1) = 0;
	
	return str;
}
/****************************************************************************/
