 !
 !  spica_struct_1A.f
 !
 !	This program reads the contents of a SPICAM or SPICAV 1A file
 !	and returns a structure containing:
 !
 !                   - "Cleandata" field: Data cleaned from :
 !                     - Missing packets (set to NaN value)
 !                     - Erroneous data (only for SPICAM data, (set to NaN value))
 !                     - Saturated pixels (saturation value is 4095 ADU, (set to NaN value))
 !                     - Impacted by comisc ray pixels ((set to NaN value))
 !                     - Electronic noise
 !                     - Dark charge
 !                     format is a 3 dimensions float array containing the formatted data. 
 !                     For a general case 5-band observation, the dimensions of
 !                     the array are (Nb_Pixel=408,Nb_Spectrum,Nb_Band=5). For an Alignment
 !                     mode observation the dimensions are (Nb_PixelX=408,Nb_PixelY=289,
 !                     Nb_Image). Window mode dimensions are (XWidth,YWidth,Nb_Record)
 !
 !                  - "Flags" field: a 3 dimensions integer array of the same dimension as 
 !                    the data fields.
 !                    Flags have the following meaning:
 !                    0: OK
 !                    1: Missing data
 !                    2: Erroneous data (SPICAM)
 !                    3: Saturated data
 !                    4: Data damaged by comisc rays
 !                    5: Data corrected from electronic noise
 !                    
 !                  - "ErrorData" field:
 !                   
 !                  - "Info" sub-structure containing information about the
 !                    input files and general observation characteristics 
 !                    (tag names in parenthesis are specific to alignment
 !                    observations):
 !                       o NAxis1[Int]         : Number of pixels of the X-axis of the 
 !                                               CCD, i.e. number of points of each spec-
 !                                               trum = 408 except for window mode.
 !                       o NAxis2[Int]         : Number of 5-band spectra recorded throug-
 !                                               hout observation. For alignment observa-
 !                                               tion this field corresponds to the number
 !                                               of pixels of the Y-axis of the CCD = 289.
 !                                               For Window modes, it is the vertical width.
 !                       o NAxis3[Int]         : Number of bands of the CCD = 5. For align-
 !                                               ment observations this is equal to the
 !                                               number of complete mapping of the CCD.
 !                                               For Window mode this is equal to the number 
 !                                               of records.
 !                       o Instrument[String]  : Instrument name.
 !                       o Orbit[Int]          : Orbit index of the mission.
 !                       o Sequence[Int]       : Sequence index of the orbit.
 !                       o ObsType[Char]       : Type of observation. Cf SPICAM/V documen
 !                                               -tation.
 !                       o BeginTime[String]   : Observation first recording time, 
 !                                               format=YYYY-MM-DDTHH:MM:SS.mSmSmS.
 !                       o EndTime[String]     : Observation last recording time, 
 !                                               format=YYYY-MM-DDTHH:MM:SS.mSmSmS
 !                       o DCProc [Boolean]    : Dark Charge treatment
 !                       o ENProc [Boolean]    : Electronic Noise Treatment
 !                       o COSProc [Boolean]   : Cosmic rays pixels Treatment
 !                       o SATProc [Boolean]   : Saturated pixels Treatment
 !                       o File0C [String]     : 0C file used to build this 1A level
 !                       o OrbDCNU [Int]       : Number of the DCNU Model orbit
 !                       o Data_status [String]: Status of the Data (F "Final" or "P" Preliminary)
 !                       o Geo_status [String] : Status of the Geometry (F "Final" or "P" Preliminary)
 !                       o Flag_status [String]: Status of the Flag (F "Final" or "P" Preliminary)
 !                       o DC_status [String]  : Status of the Dark Charge correction (F "Final" or "P" Preliminary)
 !                     
 !                 - "Parameters" sub-structure containing functional parameters 
 !                   of the instrument. For 5-bands observations the "All_Ti" and 
 !                   "T_..." fields are 1d of length the number of spectra. For 
 !                   alignment modes they are 2d with size [nb_PixelY=289,nb_Image]:
 !                       o CodeOp[Int]     : Operational code of observation. Cf SPICAM/V docu-
 !                                           mentation.
 !                       o Binning[Int]    : Number of lines binned together to form a band. In 
 !                                           case of progressive binning, the field si set to
 !                                           string 'VARIABLE'.
 !                       o HT[Int]         : Digital scale of the High Voltage used to amplify 
 !                                           signal. Ranging from 0 to 255. See SPICAM/V docu-
 !                                           mentation for corresponding voltage.
 !                       o Ti[Int]         : Exposure Time of the CCD, in second/100. Usually 
 !                                           constant throughout observation. If not the field is 
 !                                           set to string 'VARIABLE'.
 !                       o X0[Int]         : X-axis first pixel index.
 !                       o Y0[Int]         : Y-axis first pixel index. Field is set to 'VARIABLE'
 !                                           for alignment observations.
 !                       o Slit[boolean]   : 1 if slit is on, 0 else.
 !                       o Peltier[boolean]: 1 if Peltier cooler is on, 0 else.
 !                       o UVSampling[Int] : Sample rate of the spectrometer.
 !                       o IROn[boolean]   : 1 if IR channel is on, 0 else.
 !                       o SoirOn[boolean] : 1 if SPICAV SOIR channel is on, 0 else. 255 for 
 !                                           SPICAM.
 !                       o All_Ti[Int]     : vector of all the time exposure throughout obser-
 !                                           vation.
 !                       o T_Peltier[Int]  : Temperature vector of Peltier's hot side (all tem-
 !                                           peratures ar given in C).
 !                       o T_CCD[Int]      : Temperature vector of the CCD.
 !                       o T_NumBoard[Int] : Temperature vector of the digital board.
 !                       o T_BTBoard[Int]  : Temperature vector of the low tension board.
 !                       o T_Shutter[Int]  : Temperature vector of the solar shutter.
 !                       o T_ServBoard[Int]: Temperature vector of the servitudes board.
 !                       o T_HVPS[Int]     : Temperature vector of the High Voltage Power
 !                                           Supply.
 !                       o T_Structure[Int]: Temperature vector of the structure.
 !                     
 !                 - "GeoInfo" sub-structure containing geometrical parameters 
 !                   of the instrument which do not vary during observation:
 !                       o Target[String]   : Pointing target of the instrument. Cf SPICAM/V 
 !                                            documentation for the star catalog.
 !                       o SunLat[Float]    : Sun latitude planet body-fixed frame, at 
 !                                            first record (deg).
 !                       o SunLong[Float]   : Sun longitude planet body-fixed frame, at
 !                                            first record (deg).
 !                       o SunDist[ULong]   : Distance between planet and sun, at first
 !                                            record (ua).
 !                       o SunLS[Float]     : Sun solar longitude in the planet body-fixed
 !                                            frame, at first record (deg).
 !                       o SunRa[Float]     : Sun Ra, at first record (deg).
 !                       o SunDec[Float]    : Sun Dec, at first record (deg).
 !                       o SlitCenter[Float]: Position vector of the slit center in SC axes 
 !                                            (=offset, deg).
 !                       o ShadowCone[Char] : Trajectory of the spacecraft relatively to the 
 !                                            planet shadow cone. 
 !
 !                 - "Geo" sub-structure containing the time, and geometrical
 !                   parameters which vary during observation. Parameters are
 !                   grouped by the following nested structure. For 5-bands observations
 !                   the array is 1d of length the number of spectra. For alignment modes
 !                   the array is 2d with size [nb_PixelY=289,nb_Image].
 !                   NOTA: Depending on the type and mode of observations, the 
 !                   sub-structures vary. Common sub-structures are: {Record,
 !                   Spacecraft,Band3,Coordinates,TransMatrix}. All 5-band obser-
 !                   vations have the folllowing added sub-structures: {Band1,
 !                   Band2,Band4,Band5}. Star occultations have another sub-struc-
 !                   ture which is {LineOfSight}. 
 !                   Alignment modes have the specific sub-structure {CCDLine}.
 !                       o Record: + Number[Int] : observation record number, beginning at 1.
 !                                 + Time[String]: Record time, format=YYYY-MM-DDTHH:MM:SS.mSmSmS
 !                    
 !                       o Spacecraft: + Lat[Float]            : Latitude of the SC (deg).
 !                                     + Long[Float]           : Longitude of the SC (deg).
 !                                     + Alt[Float]            : Altitude of the SC in (km).
 !                                     + SZA[Float]            : Solar zenith angle of the SC (deg).
 !                                     + ShadowConeRatio[Float]: Ratio giving position of SC towards 
 !                                                               planet shadow cone. If >1 then SC is
 !                                                               outside the shadow cone else it's inside.
 !
 !                       o Band3: + Sza[Float]                : Solar zenith angle of the 3rd band/central 
 !                                                              line of the CCD portion (deg).
 !                                + DistToPlanetNP[Float]     : Distance to the planet nearest point (km).
 !                                + PixelSize[Float]          : Pixel size at planet nearest point (km).
 !                                + AngleLOSSun[Float]        : Angle between line of sight and sun (deg).
 !                                + PhaseLOSSun[Float]        : Phase between line of sight at planet near-
 !                                                              est point and sun(deg).
 !                                + SolarIncidence[Float]     : Solar incidence between normal at planet 
 !                                                              nearest point and sun (deg)
 !                                + SolarLocalTime[Float]     : Solar local time at planet nearest point (deg).
 !                                + AngleNormalObserver[Float]: Angle between Normal and Observer (deg).
 !                                + DistLOSPlanetCenter[Float]: Distance between line of sight and planet 
 !                                                              center (km).
 !                      
 !                            For non alignment modes only:
 !                                + Lat[Float] : Latitude of the 3rd Band  (deg)             
 !                                + Long[Float]: Longitude of the 3rd Band (deg)              
 !                                + Alt[Float] : Altitude of the 3rd Band (km)               
 !                                + Ra[Float]  : Ra of the 3rd Band  (deg)   
 !                                + Dec[Float] : Dec of the 3rd Band (deg)
 !
 !                       o Band1: + Lat[Float] : Latitude of the 1st Band (deg)              
 !                                + Long[Float]: Longitude of the 1st Band (deg)              
 !                                + Alt[Float] : Altitude of the 1st Band (km)               
 !                                + Ra[Float]  : Ra of the 1st Band (deg)     
 !                                + Dec[Float] : Dec of the 1st Band (deg)
 !
 !                       o Band2,Band4 and Band5 have the same corresponding fields.
 !
 !                       o CCDLine has the same corresponding fields for each line 
 !                         of the CCD. For alignment observations this structure replaces
 !                         the missing Band3 fields.
 !
 !                       o LOSE: Same fields as Band3 common fields, but associated to 
 !                               the line of sight emanating from the UV CCD center, but 
 !                               defined by S/C position and Star direction.
 ! 
 !                       o Coordinates: + Xsc_X[Float]: X coordinate of the SC X-axis in MARSIAU/IAU_VENUS.
 !                                      + Xsc_Y[Float]: Y coordinate of the SC X-axis in MARSIAU/IAU_VENUS.
 !                                      + Xsc_Z[Float]: Z coordinate of the SC X-axis in MARSIAU/IAU_VENUS.
 !                                      + Ysc_X[Float]: X coordinate of the SC Y-axis in MARSIAU/IAU_VENUS.
 !                                      + Ysc_Y[Float]: Y coordinate of the SC Y-axis in MARSIAU/IAU_VENUS.
 !                                      + Ysc_Z[Float]: Z coordinate of the SC Y-axis in MARSIAU/IAU_VENUS.
 !                                      + Zsc_X[Float]: X coordinate of the SC Z-axis in MARSIAU/IAU_VENUS.
 !                                      + Zsc_Y[Float]: Y coordinate of the SC Z-axis in MARSIAU/IAU_VENUS.
 !                                      + Zsc_Z[Float]: Z coordinate of the SC Z-axis in MARSIAU/IAU_VENUS.
 !                                      + Angle_Xsc_LocalVert[Float]: Angle between Xsc and local verti-
 !                                                                    cal at planet nearest point (deg).
 !                                      + Angle_Xsc_Proj[Float]     : Angle between Xsc and the projec-
 !                                                                    tion plan (PP) U-axis (deg).
 !                                      + P1_u[Float]: U coordinate of the horizon point in the PP.
 !                                      + P1_v[Float]: V coordinate of the horizon point in the PP.
 !                                      + P2_u[Float]: U coordinate of the horizon point in the PP.
 !                                      + P2_v[Float]: V coordinate of the horizon point in the PP.
 !                                      + P3_u[Float]: U coordinate of the horizon point in the PP.
 !                                      + P3_v[Float]: V coordinate of the horizon point in the PP.
 !                                      + P4_u[Float]: U coordinate of the horizon point in the PP.
 !                                      + P4_v[Float]: V coordinate of the horizon point in the PP.
 !                                      + P5_u[Float]: U coordinate of the horizon point in the PP.
 !                                      + P5_v[Float]: V coordinate of the horizon point in the PP.
 !
 !                       o TransMatrix: + X_Ra [Float]: Ra of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 !                                                      formation matrix X vector (deg).
 !                                      + X_Dec[Float]: Dec of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 !                                                      formation matrix X vector (deg).
 !                                      + Y_Ra [Float]: Ra of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 !                                                      formation matrix Y vector (deg).
 !                                      + Y_Dec[Float]: Dec of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 !                                                      formation matrix Y vector (deg).
 !                                      + Z_Ra [Float]: Ra of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 !                                                      formation matrix Z vector (deg).
 !                                      + Z_Dec[Float]: Dec of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
 !                                                      formation matrix Z vector (deg).
 !
 ! spica_read_1A function:
 ! Inputs:
 !      - Filename [string]: full path and name of the 1A file
 !      - Mode [Int]: output messages mode, 0:output messages disabled; 1: output messages enabled
 !
 ! Output:
 !      - File1A [struct_1Afile]: 1A file content structure, see above for details
 !
 ! Library:
 !      cfitsio: http://heasarc.gsfc.nasa.gov/fitsio/
 !      To compile this module: - Import the static library "libcfitsio.a" in your system library path
 !                              - Add the option "-I cfitsio" or "out libcfitsio.a" in you compiler command line
 !
 !  SVN Revision:
 !      $Rev: 1146 $
 !

module spica_struct_1A
    implicit none

!   Structures declaration

    TYPE :: struc_Info
        character(len=10) instrument
        character(len=1) obstype
        character(len=128) begins
        character(len=128) ends
        integer orbit
        integer seq
        integer naxis1
        integer naxis2
        integer naxis3
        integer DCProc
        integer ENProc
        integer COSProc
        integer SATProc
        character(len=128) Data_status
		character(len=128) Geo_status
		character(len=128) Flag_status
		character(len=128) DC_status

    END TYPE struc_Info

    TYPE :: struc_Parameters
        integer codeop
        integer binning
        integer ht
        integer ti
        integer x0
        integer y0
        integer slit
        integer peltier
        integer uvsampling
        integer iron
        integer soiron
        integer, dimension(:), allocatable :: all_ti
        real, dimension(:), allocatable :: t_peltier
        real, dimension(:), allocatable :: t_ccd
        real, dimension(:), allocatable :: t_numboard
        real, dimension(:), allocatable :: t_btboard
        real, dimension(:), allocatable :: t_shutter
        real, dimension(:), allocatable :: t_servboard
        real, dimension(:), allocatable :: t_hvps
        real, dimension(:), allocatable :: t_structure
    END TYPE struc_Parameters

    TYPE :: struc_GeoInfo
        character(len=16) target
        character(len=16) shadowcone
        real sunlat
        real sunlong
        real sundist
        real sunls
        real sunra
        real sundec
        real, dimension(2) :: slitcenter
    END TYPE struc_GeoInfo

    TYPE :: struc_Geo_Record
        integer, dimension(:), allocatable :: number
        character(len=128), dimension(:), allocatable :: time
    END TYPE struc_Geo_Record

    TYPE :: struc_Geo_Spacecraft
        real, dimension(:), allocatable ::  lat
        real, dimension(:), allocatable ::  long
        real, dimension(:), allocatable ::  alt
        real, dimension(:), allocatable ::  sza
        real, dimension(:), allocatable ::  shadowconeratio
    END TYPE struc_Geo_Spacecraft

    TYPE :: struc_Geo_Band3LOSE
        real, dimension(:), allocatable :: sza
        real, dimension(:), allocatable :: disttoplanetnp
        real, dimension(:), allocatable :: pixelsize
        real, dimension(:), allocatable :: anglelossun
        real, dimension(:), allocatable :: phaselossun
        real, dimension(:), allocatable :: solarincidence
        real, dimension(:), allocatable :: solarlocaltime
        real, dimension(:), allocatable :: anglenormalobserver
        real, dimension(:), allocatable :: sunazimuth
        real, dimension(:), allocatable :: distlosplanetcenter
        real, dimension(:), allocatable :: distscplanetcenter
        real, dimension(:), allocatable :: lat
        real, dimension(:), allocatable :: long
        real, dimension(:), allocatable :: alt
        real, dimension(:), allocatable :: ra
        real, dimension(:), allocatable :: dec
    END TYPE struc_Geo_Band3LOSE

    TYPE :: struc_Geo_BandCCDline
        real, dimension(:), allocatable :: lat
        real, dimension(:), allocatable :: long
        real, dimension(:), allocatable :: alt
        real, dimension(:), allocatable :: ra
        real, dimension(:), allocatable :: dec
    END TYPE struc_Geo_BandCCDline

    TYPE :: struc_Geo_Coordinates
        real, dimension(:), allocatable :: xsc_x
        real, dimension(:), allocatable :: xsc_y
        real, dimension(:), allocatable :: xsc_z
        real, dimension(:), allocatable :: ysc_x
        real, dimension(:), allocatable :: ysc_y
        real, dimension(:), allocatable :: ysc_z
        real, dimension(:), allocatable :: zsc_x
        real, dimension(:), allocatable :: zsc_y
        real, dimension(:), allocatable :: zsc_z
        real, dimension(:), allocatable :: angle_xsc_localvert
        real, dimension(:), allocatable :: angle_xsc_proj
        real, dimension(:), allocatable :: p1_u
        real, dimension(:), allocatable :: p1_v
        real, dimension(:), allocatable :: p2_u
        real, dimension(:), allocatable :: p2_v
        real, dimension(:), allocatable :: p3_u
        real, dimension(:), allocatable :: p3_v
        real, dimension(:), allocatable :: p4_u
        real, dimension(:), allocatable :: p4_v
        real, dimension(:), allocatable :: p5_u
        real, dimension(:), allocatable :: p5_v
    END TYPE struc_Geo_Coordinates

    TYPE :: struc_Geo_TransMatrix
        real, dimension(:), allocatable :: x_ra
        real, dimension(:), allocatable :: x_dec
        real, dimension(:), allocatable :: y_ra
        real, dimension(:), allocatable :: y_dec
        real, dimension(:), allocatable :: z_ra
        real, dimension(:), allocatable :: z_dec
    END TYPE struc_Geo_TransMatrix

    TYPE :: struc_Geo
        type(struc_Geo_Record) Record
        type(struc_Geo_Spacecraft) Spacecraft
        type(struc_Geo_Band3LOSE) Band3
        type(struc_Geo_Band3LOSE) LOSE
        type(struc_Geo_BandCCDline) Band1
        type(struc_Geo_BandCCDline) Band2
        type(struc_Geo_BandCCDline) Band4
        type(struc_Geo_BandCCDline) Band5
        type(struc_Geo_BandCCDline) CCDLine
        type(struc_Geo_Coordinates) Coordinates
        type(struc_Geo_TransMatrix) TransMatrix
    END TYPE struc_Geo

    TYPE :: struc_1AFile
        real, dimension(:,:,:), allocatable :: CleanData
        real, dimension(:,:,:), allocatable :: ErrData
        integer, dimension(:,:,:), allocatable :: Flag
        type(struc_Info) Info
        type(struc_Parameters) Parameters
        type(struc_GeoInfo) GeoInfo
        type(struc_Geo) Geo
    END TYPE struc_1AFile;

contains
    ! Read primary header
    type(struc_Info) function readInfo(unit_,status)
        integer :: unit_,status,status_all
        character(len=80) :: comm
        type(struc_Info) :: Info

        status_all = 0

        call ftgkys(unit_,'INSTRU',Info%Instrument,comm,status)
        status_all = status_all + status
        call ftgkys(unit_,'OBSTYPE',Info%obstype,comm,status)
        status_all = status_all + status
        call ftgkys(unit_,'BEGINS',Info%begins,comm,status)
        status_all = status_all + status
        call ftgkys(unit_,'ENDS',Info%ends,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'ORBIT',Info%orbit,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'SEQ_NB',Info%seq,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'NAXIS1',Info%naxis1,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'NAXIS2',Info%naxis2,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'NAXIS3',Info%naxis3,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'DCProc',Info%DCProc,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'ENProc',Info%ENProc,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'COSProc',Info%COSProc,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'SATProc',Info%SATProc,comm,status)
        status_all = status_all + status
        call ftgkys(unit_,'DATA_SS',Info%Data_status,comm,status)
        status_all = status_all + status
        call ftgkys(unit_,'GEO_SS',Info%Geo_status,comm,status)
        status_all = status_all + status
        call ftgkys(unit_,'FLAG_SS',Info%Flag_status,comm,status)
        status_all = status_all + status
	    call ftgkys(unit_,'DC_SS',Info%DC_status,comm,status)
        status_all = status_all + status
		
        status = status_all

        readInfo = Info

    end function readInfo

    ! Read Functional_Parameters
    type(struc_Parameters) function readParameters(unit_,status)
        integer :: unit_,err,status,status_all,nullval,frow,felem,naxis2,err_alloc
        character(len=80) :: comm
        character(len=30) :: errmsg
        logical anyf
        type(struc_Parameters) :: Parameters

        ! Init
        status_all = 0
        err_alloc = 0
        nullval = 0
        frow = 1
        felem = 1

        ! Move to the HDU
        call ftmnhd(unit_,2,'Functional_Parameters',0,status) !ftmhd type 2 = bin table
        status_all = status_all + status
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readParameters: '//errmsg
        endif

        ! Array Allocation
        call ftgkyj(unit_,'NAXIS2',naxis2,comm,status)
        allocate(Parameters%all_ti(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Parameters%t_peltier(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Parameters%t_ccd(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Parameters%t_numboard(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Parameters%t_btboard(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Parameters%t_shutter(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Parameters%t_servboard(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Parameters%t_hvps(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Parameters%t_structure(naxis2),stat=err)
        err_alloc = err_alloc + err

        if(err_alloc .ne. 0) then
            print  *,'readGeo: Array allocation error'
            call exit()
        endif

        ! Read parameters
        call ftgkyj(unit_,'CODEOP',Parameters%codeop,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'BINNING',Parameters%binning,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'TI',Parameters%ti,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'HT',Parameters%ht,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'X0',Parameters%x0,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'Y0',Parameters%y0,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'SLIT',Parameters%slit,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'PELTIER',Parameters%peltier,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'UVSAMPL',Parameters%uvsampling,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'IR_ON',Parameters%iron,comm,status)
        status_all = status_all + status
        call ftgkyj(unit_,'SOIR_ON',Parameters%soiron,comm,status)
        status_all = status_all + status

        ! Read table data
        call ftgcvj(unit_,1,frow,felem,naxis2,nullval,Parameters%all_ti,anyf,status)
        status_all = status_all + status
        call ftgcve(unit_,2,frow,felem,naxis2,nullval,Parameters%t_peltier,anyf,status)
        status_all = status_all + status
        call ftgcve(unit_,3,frow,felem,naxis2,nullval,Parameters%t_ccd,anyf,status)
        status_all = status_all + status
        call ftgcve(unit_,4,frow,felem,naxis2,nullval,Parameters%t_numboard,anyf,status)
        status_all = status_all + status
        call ftgcve(unit_,5,frow,felem,naxis2,nullval,Parameters%t_btboard,anyf,status)
        status_all = status_all + status
        call ftgcve(unit_,6,frow,felem,naxis2,nullval,Parameters%t_shutter,anyf,status)
        status_all = status_all + status
        call ftgcve(unit_,7,frow,felem,naxis2,nullval,Parameters%t_servboard,anyf,status)
        status_all = status_all + status
        call ftgcve(unit_,8,frow,felem,naxis2,nullval,Parameters%t_hvps,anyf,status)
        status_all = status_all + status
        call ftgcve(unit_,9,frow,felem,naxis2,nullval,Parameters%t_structure,anyf,status)
        status_all = status_all + status

        status = status_all

        readParameters = Parameters

    end function readParameters

    ! Read GEO Infos
    type(struc_GeoInfo) function readGeoInfo(unit_,status)
        integer :: unit_, status, pos, status_all
        character(len=80) :: comm
        character(len=16) :: slitc
        character(len=30) :: errmsg
        type(struc_GeoInfo) :: GeoInfo

        status_all = 0

        ! Move to the HDU
        call ftmnhd(unit_,2,'Geo_Record',0,status) !ftmhd type 2 = bin table
        status_all = status_all + status
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readGeoInfo: '//errmsg
        endif

        call ftgkys(unit_,'TARGET',GeoInfo%target,comm,status)
        status_all = status_all + status
        call ftgkys(unit_,'CONE',GeoInfo%shadowcone,comm,status)
        status_all = status_all + status
        call ftgkys(unit_,'SLIT_C',slitc,comm,status)
        status_all = status_all + status
        call ftgkye(unit_,'SUNLAT',GeoInfo%sunlat,comm,status)
        status_all = status_all + status
        call ftgkye(unit_,'SUNLONG',GeoInfo%sunlong,comm,status)
        status_all = status_all + status
        call ftgkye(unit_,'SUNDIST',GeoInfo%sundist,comm,status)
        status_all = status_all + status
        call ftgkye(unit_,'SUNLS',GeoInfo%sunls,comm,status)
        status_all = status_all + status
        call ftgkye(unit_,'SUNRA',GeoInfo%sunra,comm,status)
        status_all = status_all + status
        call ftgkye(unit_,'SUNDEC',GeoInfo%sundec,comm,status)
        status_all = status_all + status

        pos = index(slitc," ",.true.)
        read(slitc(1:pos),*) GeoInfo%slitcenter(1)
        read(slitc(pos:len(slitc)),*) GeoInfo%slitcenter(2)

        status = status_all

        readGeoInfo = GeoInfo

    end function readGeoInfo

    ! Read GEO Data
    type(struc_Geo) function readGeo(unit_,status,codeop)
        integer :: unit_,err,err_alloc,status
        integer :: status_record,status_spacecraft,status_band1,status_band2 &
                   ,status_band3,status_band4,status_band5,status_ccdline &
                   ,status_lose,status_coord,status_trans
        integer :: naxis2,frow,felem,codeop
        real :: nullval,anyf
        character nullval_s
        character(len=80) :: comm
        character(len=30) :: errmsg
        type(struc_Geo_Record) :: Geo_Record
        type(struc_Geo_Spacecraft) :: Geo_Spacecraft
        type(struc_Geo_Band3LOSE) :: Geo_Band3
        type(struc_Geo_BandCCDline) :: Geo_Band1
        type(struc_Geo_BandCCDline) :: Geo_Band2
        type(struc_Geo_BandCCDline) :: Geo_Band4
        type(struc_Geo_BandCCDline) :: Geo_Band5
        type(struc_Geo_BandCCDline) :: Geo_CCDLine
        type(struc_Geo_Band3LOSE) :: Geo_LOSE
        type(struc_Geo_Coordinates) :: Geo_Coordinates
        type(struc_Geo_TransMatrix) :: Geo_TransMatrix
        type(struc_Geo) :: Geo

        !!! Init
        status_record = 0
        status_spacecraft = 0
        status_band1 = 0
        status_band2 = 0
        status_band3 = 0
        status_band4 = 0
        status_band5 = 0
        status_ccdline = 0
        status_lose = 0
        status_coord = 0
        status_trans = 0

        err_alloc = 0
        anyf = 0
        nullval = 0
        nullval_s = '0'
        frow = 1
        felem = 1

        !!! Move the HDU to the Geo_record binary_table
        call ftmnhd(unit_,2,'Geo_Record',0,status) !ftmhd type 2 = bin table
        status_record = status_record + status
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readGeo: '//errmsg
        endif

        ! Array Allocation
        call ftgkyj(unit_,'NAXIS2',naxis2,comm,status)
        allocate(Geo_Record%number(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Record%time(naxis2),stat=err)
        err_alloc = err_alloc + err

        if(err_alloc .ne. 0) then
            print  *,'readGeo: Array allocation error'
            call exit()
        endif

        ! Read table data
        call ftgcvs(unit_,1,1,1,naxis2,nullval_s,Geo_Record%time,anyf,status)
        status_record = status_record + status
        call ftgcvj(unit_,2,frow,felem,naxis2,nullval,Geo_Record%number,anyf,status)
        status_record = status_record + status

        !!! Move the HDU to the Geo_Spacecraft binary_table
        call ftmnhd(unit_,2,'Geo_Spacecraft',0,status) !ftmhd type 2 = bin table
        status_spacecraft = status_spacecraft + status
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readGeo: '//errmsg
        endif

        ! Array Allocation
        allocate(Geo_Spacecraft%lat(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Spacecraft%long(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Spacecraft%alt(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Spacecraft%sza(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Spacecraft%shadowconeratio(naxis2),stat=err)
        err_alloc = err_alloc + err

        if(err_alloc .ne. 0) then
            print  *,'readGeo: Array allocation error'
            call exit()
        endif

        ! Read table data
        call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_Spacecraft%lat,anyf,status)
        status_spacecraft = status_spacecraft + status
        call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_Spacecraft%long,anyf,status)
        status_spacecraft = status_spacecraft + status
        call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_Spacecraft%alt,anyf,status)
        status_spacecraft = status_spacecraft + status
        call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_Spacecraft%sza,anyf,status)
        status_spacecraft = status_spacecraft + status
        call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_Spacecraft%shadowconeratio,anyf,status)
        status_spacecraft = status_spacecraft + status

        !!! Move the HDU to the Geo_Band3 binary_table
        call ftmnhd(unit_,2,'Geo_Band3',0,status) !ftmhd type 2 = bin table
        status_band3 = status_band3 + status
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readGeo: '//errmsg
        endif

        ! Array Allocation
        allocate(Geo_Band3%sza(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%disttoplanetnp(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%pixelsize(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%anglelossun(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%phaselossun(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%solarincidence(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%solarlocaltime(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%anglenormalobserver(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%sunazimuth(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%distlosplanetcenter(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%distscplanetcenter(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%lat(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%long(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%alt(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band3%dec(naxis2),stat=err)
        err_alloc = err_alloc + err

        if(err_alloc .ne. 0) then
            print  *,'readGeo: Array allocation error'
            call exit()
        endif

        ! Read table data
        if(codeop .ge. 100) then
            call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_Band3%lat,anyf,status)
            status_band3 = status_band3 + status
            call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_Band3%long,anyf,status)
            status_band3 = status_band3 + status
            call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_Band3%alt,anyf,status)
            status_band3 = status_band3 + status
            call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_Band3%ra,anyf,status)
            status_band3 = status_band3 + status
            call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_Band3%dec,anyf,status)
            status_band3 = status_band3 + status
        else
            Geo_Band3%lat = 0
            Geo_Band3%long = 0
            Geo_Band3%alt = 0
            Geo_Band3%ra = 0
            Geo_Band3%dec = 0
        endif

        call ftgcve(unit_,6,frow,felem,naxis2,nullval,Geo_Band3%sza,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,7,frow,felem,naxis2,nullval,Geo_Band3%disttoplanetnp,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,8,frow,felem,naxis2,nullval,Geo_Band3%pixelsize,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,9,frow,felem,naxis2,nullval,Geo_Band3%anglelossun,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,10,frow,felem,naxis2,nullval,Geo_Band3%phaselossun,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,11,frow,felem,naxis2,nullval,Geo_Band3%solarincidence,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,12,frow,felem,naxis2,nullval,Geo_Band3%solarlocaltime,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,13,frow,felem,naxis2,nullval,Geo_Band3%anglenormalobserver,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,14,frow,felem,naxis2,nullval,Geo_Band3%sunazimuth,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,15,frow,felem,naxis2,nullval,Geo_Band3%distlosplanetcenter,anyf,status)
        status_band3 = status_band3 + status
        call ftgcve(unit_,16,frow,felem,naxis2,nullval,Geo_Band3%distscplanetcenter,anyf,status)
        status_band3 = status_band3 + status

        !!! Geo_Band1,2,4 et 5 or CCDLine binary_table, depending on codeop
        ! Array Allocation
        allocate(Geo_Band1%lat(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band1%long(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band1%alt(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band1%ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band1%dec(naxis2),stat=err)
        err_alloc = err_alloc + err

        allocate(Geo_Band2%lat(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band2%long(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band2%alt(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band2%ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band2%dec(naxis2),stat=err)
        err_alloc = err_alloc + err

        allocate(Geo_Band4%lat(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band4%long(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band4%alt(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band4%ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band4%dec(naxis2),stat=err)
        err_alloc = err_alloc + err

        allocate(Geo_Band5%lat(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band5%long(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band5%alt(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band5%ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Band5%dec(naxis2),stat=err)
        err_alloc = err_alloc + err

        allocate(Geo_CCDLine%lat(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_CCDLine%long(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_CCDLine%alt(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_CCDLine%ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_CCDLine%dec(naxis2),stat=err)
        err_alloc = err_alloc + err

        if(err_alloc .ne. 0) then
            print  *,'readGeo: Array allocation error'
            call exit()
        endif

        if(codeop .ge. 100) then
            !! Move the HDU to the Geo_Band1 binary table
            call ftmnhd(unit_,2,'Geo_Band1',0,status) !ftmhd type 2 = bin table
            status_band1 = status_band1 + status
            if(status .ne. 0) then
                call ftgerr(status,errmsg)
                print *,'readGeo: '//errmsg
            endif

            ! Read data
            call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_Band1%lat,anyf,status)
            status_band1 = status_band1 + status
            call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_Band1%long,anyf,status)
            status_band1 = status_band1 + status
            call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_Band1%alt,anyf,status)
            status_band1 = status_band1 + status
            call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_Band1%ra,anyf,status)
            status_band1 = status_band1 + status
            call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_Band1%dec,anyf,status)
            status_band1 = status_band1 + status

            !! Move the HDU to the Geo_Band2 binary table
            call ftmnhd(unit_,2,'Geo_Band2',0,status) !ftmhd type 2 = bin table
            status_band2 = status_band2 + status
            if(status .ne. 0) then
                call ftgerr(status,errmsg)
                print *,'readGeo: '//errmsg
            endif

            ! Read data
            call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_Band2%lat,anyf,status)
            status_band2 = status_band2 + status
            call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_Band2%long,anyf,status)
            status_band2 = status_band2 + status
            call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_Band2%alt,anyf,status)
            status_band2 = status_band2 + status
            call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_Band2%ra,anyf,status)
            status_band2 = status_band2 + status
            call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_Band2%dec,anyf,status)
            status_band2 = status_band2 + status

            !! Move the HDU to the Geo_Band4 binary table
            call ftmnhd(unit_,2,'Geo_Band4',0,status) !ftmhd type 2 = bin table
            status_band4 = status_band4 + status
            if(status .ne. 0) then
                call ftgerr(status,errmsg)
                print *,'readGeo: '//errmsg
            endif

            ! Read data
            call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_Band4%lat,anyf,status)
            status_band4 = status_band4 + status
            call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_Band4%long,anyf,status)
            status_band4 = status_band4 + status
            call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_Band4%alt,anyf,status)
            status_band4 = status_band4 + status
            call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_Band4%ra,anyf,status)
            status_band4 = status_band4 + status
            call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_Band4%dec,anyf,status)
            status_band4 = status_band4 + status

            !! Move the HDU to the Geo_Band5 binary table
            call ftmnhd(unit_,2,'Geo_Band5',0,status) !ftmhd type 2 = bin table
            status_band5 = status_band5 + status
            if(status .ne. 0) then
                call ftgerr(status,errmsg)
                print *,'readGeo: '//errmsg
            endif

            ! Read data
            call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_Band5%lat,anyf,status)
            status_band5 = status_band5 + status
            call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_Band5%long,anyf,status)
            status_band5 = status_band5 + status
            call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_Band5%alt,anyf,status)
            status_band5 = status_band5 + status
            call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_Band5%ra,anyf,status)
            status_band5 = status_band5 + status
            call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_Band5%dec,anyf,status)
            status_band5 = status_band5 + status

            ! Set values to 0 for Geo_CCDLine
            Geo_CCDLine%lat = 0
            Geo_CCDLine%long = 0
            Geo_CCDLine%alt = 0
            Geo_CCDLine%ra = 0
            Geo_CCDLine%dec = 0
            status_ccdline = 0
        else
            !! Move the HDU to the Geo_CCDLine binary table
            call ftmnhd(unit_,2,'Geo_CCDLine',0,status) !ftmhd type 2 = bin table
            status_ccdline = status_ccdline + status
            if(status .ne. 0) then
                call ftgerr(status,errmsg)
                print *,'readGeo: '//errmsg
            endif

            ! Read data
            call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_CCDLine%lat,anyf,status)
            status_ccdline = status_ccdline + status
            call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_CCDLine%long,anyf,status)
            status_ccdline = status_ccdline + status
            call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_CCDLine%alt,anyf,status)
            status_ccdline = status_ccdline + status
            call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_CCDLine%ra,anyf,status)
            status_ccdline = status_ccdline + status
            call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_CCDLine%dec,anyf,status)
            status_ccdline = status_ccdline + status

            ! Set values to 0 for the other bands
            Geo_Band1%lat = 0
            Geo_Band1%long = 0
            Geo_Band1%alt = 0
            Geo_Band1%ra = 0
            Geo_Band1%dec = 0
            status_band1 = 0

            Geo_Band2%lat = 0
            Geo_Band2%long = 0
            Geo_Band2%alt = 0
            Geo_Band2%ra = 0
            Geo_Band2%dec = 0
            status_band1 = 0

            Geo_Band4%lat = 0
            Geo_Band4%long = 0
            Geo_Band4%alt = 0
            Geo_Band4%ra = 0
            Geo_Band4%dec = 0
            status_band1 = 0

            Geo_Band5%lat = 0
            Geo_Band5%long = 0
            Geo_Band5%alt = 0
            Geo_Band5%ra = 0
            Geo_Band5%dec = 0
            status_band1 = 0
        endif

        !! If existing, read LOSE data, set to 0 otherwise
        ! Array Allocation
        allocate(Geo_LOSE%sza(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%disttoplanetnp(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%pixelsize(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%anglelossun(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%phaselossun(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%solarincidence(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%solarlocaltime(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%anglenormalobserver(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%sunazimuth(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%distlosplanetcenter(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%distscplanetcenter(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%lat(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%long(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%alt(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_LOSE%dec(naxis2),stat=err)
        err_alloc = err_alloc + err

        if(err_alloc .ne. 0) then
            print  *,'readGeo: Array allocation error'
            call exit()
        endif

        !! Move the HDU to the Geo_LOSE binary table, if existing
        call ftmnhd(unit_,2,'Geo_LOSE',0,status) !ftmhd type 2 = bin table
        if(status .eq. 0) then

            ! Read data
            call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_LOSE%lat,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_LOSE%long,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_LOSE%alt,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_LOSE%ra,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_LOSE%dec,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,6,frow,felem,naxis2,nullval,Geo_LOSE%sza,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,7,frow,felem,naxis2,nullval,Geo_LOSE%disttoplanetnp,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,8,frow,felem,naxis2,nullval,Geo_LOSE%pixelsize,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,9,frow,felem,naxis2,nullval,Geo_LOSE%anglelossun,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,10,frow,felem,naxis2,nullval,Geo_LOSE%phaselossun,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,11,frow,felem,naxis2,nullval,Geo_LOSE%solarincidence,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,12,frow,felem,naxis2,nullval,Geo_LOSE%solarlocaltime,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,13,frow,felem,naxis2,nullval,Geo_LOSE%anglenormalobserver,anyf,status)
            status_lose = status_lose + status
            call ftgcve(unit_,14,frow,felem,naxis2,nullval,Geo_LOSE%sunazimuth,anyf,status)
            status_lose = status_lose + status
            !call ftgcve(unit_,15,frow,felem,naxis2,nullval,Geo_LOSE%distlosplanetcenter,anyf,status)
            !status_lose = status_lose + status
            !call ftgcve(unit_,16,frow,felem,naxis2,nullval,Geo_LOSE%distscplanetcenter,anyf,status)
            !status_lose = status_lose + status
            Geo_LOSE%distlosplanetcenter = 0
            Geo_LOSE%distscplanetcenter = 0
        else
            Geo_LOSE%lat = 0
            Geo_LOSE%long = 0
            Geo_LOSE%alt = 0
            Geo_LOSE%ra = 0
            Geo_LOSE%dec = 0
            Geo_LOSE%sza = 0
            Geo_LOSE%disttoplanetnp = 0
            Geo_LOSE%pixelsize = 0
            Geo_LOSE%anglelossun = 0
            Geo_LOSE%phaselossun = 0
            Geo_LOSE%solarincidence = 0
            Geo_LOSE%solarlocaltime = 0
            Geo_LOSE%anglenormalobserver = 0
            Geo_LOSE%sunazimuth = 0
            Geo_LOSE%distlosplanetcenter = 0
            Geo_LOSE%distscplanetcenter = 0
            status_lose = 0
        endif
        status = 0 ! reset status

        !!! Move the HDU to the Geo_Coordinates binary_table
        call ftmnhd(unit_,2,'Geo_Coordinates',0,status) !ftmhd type 2 = bin table
        status_coord = status_coord + status
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readGeo: '//errmsg
        endif

        ! Array Allocation
        allocate(Geo_Coordinates%xsc_x(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%xsc_y(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%xsc_z(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%ysc_x(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%ysc_y(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%ysc_z(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%zsc_x(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%zsc_y(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%zsc_z(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%angle_xsc_localvert(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%angle_xsc_proj(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p1_u(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p1_v(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p2_u(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p2_v(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p3_u(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p3_v(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p4_u(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p4_v(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p5_u(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_Coordinates%p5_v(naxis2),stat=err)
        err_alloc = err_alloc + err

        if(err_alloc .ne. 0) then
            print  *,'readGeo: Array allocation error'
            call exit()
        endif

        ! Read data
        call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_Coordinates%xsc_x,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_Coordinates%xsc_y,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_Coordinates%xsc_z,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_Coordinates%ysc_x,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_Coordinates%ysc_y,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,6,frow,felem,naxis2,nullval,Geo_Coordinates%ysc_z,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,7,frow,felem,naxis2,nullval,Geo_Coordinates%zsc_x,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,8,frow,felem,naxis2,nullval,Geo_Coordinates%zsc_y,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,9,frow,felem,naxis2,nullval,Geo_Coordinates%zsc_z,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,10,frow,felem,naxis2,nullval,Geo_Coordinates%angle_xsc_localvert,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,11,frow,felem,naxis2,nullval,Geo_Coordinates%angle_xsc_proj,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,12,frow,felem,naxis2,nullval,Geo_Coordinates%p1_u,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,13,frow,felem,naxis2,nullval,Geo_Coordinates%p1_v,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,14,frow,felem,naxis2,nullval,Geo_Coordinates%p2_u,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,15,frow,felem,naxis2,nullval,Geo_Coordinates%p2_v,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,16,frow,felem,naxis2,nullval,Geo_Coordinates%p3_u,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,17,frow,felem,naxis2,nullval,Geo_Coordinates%p3_v,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,18,frow,felem,naxis2,nullval,Geo_Coordinates%p4_u,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,19,frow,felem,naxis2,nullval,Geo_Coordinates%p4_v,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,20,frow,felem,naxis2,nullval,Geo_Coordinates%p5_u,anyf,status)
        status_coord = status_coord + status
        call ftgcve(unit_,21,frow,felem,naxis2,nullval,Geo_Coordinates%p5_v,anyf,status)
        status_coord = status_coord + status

        !!! Move the HDU to the Geo_Coordinates binary_table
        call ftmnhd(unit_,2,'Geo_TransMatrix',0,status) !ftmhd type 2 = bin table
        status_trans = status_trans + status
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readGeo: '//errmsg
        endif

        ! Array Allocation
        allocate(Geo_TransMatrix%x_ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_TransMatrix%x_dec(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_TransMatrix%y_ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_TransMatrix%y_dec(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_TransMatrix%z_ra(naxis2),stat=err)
        err_alloc = err_alloc + err
        allocate(Geo_TransMatrix%z_dec(naxis2),stat=err)
        err_alloc = err_alloc + err

        if(err_alloc .ne. 0) then
            print  *,'readGeo: Array allocation error'
            call exit()
        endif

        call ftgcve(unit_,1,frow,felem,naxis2,nullval,Geo_TransMatrix%x_ra,anyf,status)
        status_trans = status_trans + status
        call ftgcve(unit_,2,frow,felem,naxis2,nullval,Geo_TransMatrix%x_dec,anyf,status)
        status_trans = status_trans + status
        call ftgcve(unit_,3,frow,felem,naxis2,nullval,Geo_TransMatrix%y_ra,anyf,status)
        status_trans = status_trans + status
        call ftgcve(unit_,4,frow,felem,naxis2,nullval,Geo_TransMatrix%y_dec,anyf,status)
        status_trans = status_trans + status
        call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_TransMatrix%z_ra,anyf,status)
        status_trans = status_trans + status
        call ftgcve(unit_,5,frow,felem,naxis2,nullval,Geo_TransMatrix%z_dec,anyf,status)
        status_trans = status_trans + status

        ! Fill Geo structure
        Geo%Record = Geo_Record
        Geo%Spacecraft = Geo_Spacecraft
        Geo%Band3 = Geo_Band3
        Geo%LOSE = Geo_LOSE
        Geo%Band1 = Geo_Band1
        Geo%Band2 = Geo_Band2
        Geo%Band4 = Geo_Band4
        Geo%Band5 = Geo_Band5
        Geo%CCDLine = Geo_CCDLine
        Geo%Coordinates = Geo_Coordinates
        Geo%TransMatrix = Geo_TransMatrix

        status = status_record + status_spacecraft + status_band1 + status_band2 &
                   + status_band3 + status_band4 + status_band5 + status_ccdline &
                   + status_lose + status_coord + status_trans

        readGeo = Geo

    end function readGeo

    ! Read Clean Data
    subroutine readCleanData(unit_,status,Naxis1,Naxis2,Naxis3,Data)
        integer :: unit_,status,type,status_all
        integer :: Naxis1,Naxis2,Naxis3
        real :: nullval = 0, anyf=0
        real, dimension(Naxis1,Naxis2,Naxis3) :: Data
        character(len=30) errmsg

        ! Read the primary Data
        call ftmahd(unit_,1,type,status)
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readFlag: '//errmsg
        endif
        status_all = status

        call ftg3de(unit_,1,nullval,Naxis1,Naxis2,Naxis1,Naxis2,Naxis3,Data,anyf,status)
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readData: '//errmsg
        endif
        status_all = status_all + status

        status = status_all

    end subroutine readCleanData

    ! Read Image extension Flag
    subroutine readFlag(unit_,status,Naxis1,Naxis2,Naxis3,Flag)
        integer :: unit_,status,status_all
        integer :: Naxis1,Naxis2,Naxis3
        integer, dimension(Naxis1,Naxis2,Naxis3) :: Flag
        real :: nullval = 0, anyf=0
        character(len=30) errmsg

        ! Move the HDU to the Flag image extension
        call ftmnhd(unit_,0,'Flag',0,status) !ftmhd type 0 = img extension
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readFlag: '//errmsg
        endif
        status_all = status

        ! Read the image extension
        call ftg3dj(unit_,1,nullval,Naxis1,Naxis2,Naxis1,Naxis2,Naxis3,Flag,anyf,status)
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readFlag: '//errmsg
        endif
        status_all = status_all + status

        status = status_all

    end subroutine readFlag

    ! Read Image extension ErrData
    subroutine readErrData(unit_,status,Naxis1,Naxis2,Naxis3,ErrData)
        integer :: unit_,status,status_all
        integer :: Naxis1,Naxis2,Naxis3
        real, dimension(Naxis1,Naxis2,Naxis3) :: ErrData
        real :: nullval = 0, anyf=0
        character(len=30) errmsg

        ! Move the HDU to the Flag image extension
        call ftmnhd(unit_,0,'ErrData',0,status) !ftmhd type 0 = img extension
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readErrData: '//errmsg
        endif
        status_all = status

        ! Read the image extension
        call ftg3de(unit_,1,nullval,Naxis1,Naxis2,Naxis1,Naxis2,Naxis3,ErrData,anyf,status)
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,'readErrData: '//errmsg
        endif
        status_all = status_all + status

        status = status_all

    end subroutine readErrData

    ! Get the basename  of a filename
    character(len=32) function file_basename(fname)
        integer pos
        character(len=2048) fname

        pos = index(fname,"/",.true.)
        file_basename = trim(fname(pos+1:len(fname)))

    end function file_basename


    type(struc_1AFile) function spica_read_1A(filename,mode)
        integer :: unit,readwrite,err,status, blocksize, mode
        integer :: check_clean, check_err, check_flag, check_info, &
                    check_param, check_geoinfo, check_geo, check_all
        character(len=2048) filename
        character(len=30) errmsg
        type(struc_Info) :: Info
        type(struc_Parameters) :: Parameters
        type(struc_GeoInfo) :: GeoInfo
        type(struc_Geo) :: Geo
        type(struc_1AFile) :: File1A
        real, dimension(:,:,:), allocatable :: CleanData
        real, dimension(:,:,:), allocatable :: ErrData
        integer, dimension(:,:,:), allocatable :: Flag

        check_info = 1
        check_param = 1
        check_geoinfo = 1
        check_clean = 1
        check_err = 1
        check_flag = 1
        check_geo = 1

    !   Open the FITS file
        call ftgiou(unit,status)
        readwrite=0
        call ftopen(unit,filename,readwrite,blocksize,status)
        if(status .ne. 0) then
            call ftgerr(status,errmsg)
            print *,errmsg
        else
            Info = readInfo(unit,status)
            if(status .eq. 0) then
                if(mode .ne. 0) then
                    print *,'Info successfully read'
                endif
                File1A%Info = Info
                check_info = 0
            endif

            Parameters = readParameters(unit,status)
            if(status .eq. 0) then
                if(mode .ne. 0) then
                    print *,'Parameters successfully read'
                endif
                File1A%Parameters = Parameters
                check_param = 0
            endif

            !!! Geo Info
            GeoInfo = readGeoInfo(unit,status)
            if(status .eq. 0) then
                if(mode .ne. 0) then
                    print *,'Geo Info successfully read'
                endif
                File1A%GeoInfo = GeoInfo
                check_geoinfo = 0
            endif

            !!! Geo Data
            Geo = readGeo(unit,status,Parameters%codeop)
            if(status .eq. 0) then
                if(mode .ne. 0) then
                    print *,'Geo Data successfully read'
                endif
                File1A%Geo = Geo
                check_geo = 0
            endif

            !!! Clean Data
            allocate(CleanData(Info%naxis1,Info%naxis2,Info%naxis3),stat=err)
            if(err .ne. 0) then
                print  *,'Array allocation error'
                call exit()
            endif

            allocate(File1A%CleanData(Info%naxis1,Info%naxis2,Info%naxis3),stat=err)
            if(err .ne. 0) then
                print  *,'Array allocation error'
                call exit()
            endif

            call readCleanData(unit,status,Info%naxis1,Info%naxis2,Info%naxis3,CleanData)
            if(status .eq. 0) then
                if(mode .ne. 0) then
                    print *,'Clean Data successfully read'
                endif
                File1A%CleanData = CleanData
                check_clean = 0
            endif

            !!! Error Data
            allocate(ErrData(Info%naxis1,Info%naxis2,Info%naxis3),stat=err)
            if(err .ne. 0) then
                print  *,'Array allocation error'
                call exit()
            endif

            allocate(File1A%ErrData(Info%naxis1,Info%naxis2,Info%naxis3),stat=err)
            if(err .ne. 0) then
                print  *,'Array allocation error'
                call exit()
            endif

            call readErrData(unit,status,Info%naxis1,Info%naxis2,Info%naxis3,ErrData)
            if(status .eq. 0) then
                if(mode .ne. 0) then
                    print *,'Err Data successfully read'
                endif
                File1A%ErrData = ErrData
                check_err = 0
            endif

            !!! Flag Data
            allocate(FLag(Info%naxis1,Info%naxis2,Info%naxis3),stat=err)
            if(err .ne. 0) then
                print  *,'Array allocation error'
                call exit()
            endif

            allocate(File1A%FLag(Info%naxis1,Info%naxis2,Info%naxis3),stat=err)
            if(err .ne. 0) then
                print  *,'Array allocation error'
                call exit()
            endif

            call readFlag(unit,status,Info%naxis1,Info%naxis2,Info%naxis3,Flag)
            if(status .eq. 0) then
                if(mode .ne. 0) then
                    print *,'Flag Data successfully read'
                endif
                File1A%FLag = FLag
                check_flag = 0
            endif

            check_all = check_info + check_param + check_geoinfo + check_clean + check_geo + check_err

            if(check_all .eq. 0 .and. mode .ne. 0) then
                print *,'The file '//file_basename(filename)//'was read succesfully!'
            endif

            call ftclos(unit, status)
            call ftfiou(unit, status)

            spica_read_1A = File1A
        endif ! ftopen

    end function spica_read_1A

end module spica_struct_1A
