;+
; NAME: 
;       SPICA_Read_1A
;
; PURPOSE:    
;       Read a SPICAM/V Data 1A file and return a structure with a 3d 
;       data array and sub-structures containing functional and geometrical
;       parameters information of the instrument during observation. 
;
; CALLING SEQUENCE: 
;       Result   = SPICA_Read_1A(Filename,/SILENT)
;
; INPUTS:
;       Filename = String containing the full Data 1A type file name (FITS format).
;
; OPT. INPUTS: 
;       None.
;
; OUTPUTS: 
;       Result   = Structure containing:
;
;                   - "Cleandata" field: Data cleaned from :
;                     - Missing packets (set to NaN value)
;                     - Erroneous data (only for SPICAM data, (set to NaN value))
;                     - Saturated pixels (saturation value is 4095 ADU, (set to NaN value))
;                     - Impacted by comisc ray pixels ((set to NaN value))
;                     - Electronic noise
;                     - Dark charge
;                     format is a 3 dimensions float array containing the formatted data. 
;                     For a general case 5-band observation, the dimensions of
;                     the array are (Nb_Pixel=408,Nb_Spectrum,Nb_Band=5). For an Alignment
;                     mode observation the dimensions are (Nb_PixelX=408,Nb_PixelY=289,
;                     Nb_Image). Window mode dimensions are (XWidth,YWidth,Nb_Record)
;
;                  - "Flags" field: a 3 dimensions integer array of the same dimension as 
;                    the data fields.
;                    Flags have the following meaning:
;                    0: OK
;                    1: Missing data
;                    2: Erroneous data (SPICAM)
;                    3: Saturated data
;                    4: Data damaged by comisc rays
;                    5: Data corrected from electronic noise
;                    
;                  - "ErrorData" field:
;                   
;                  - "Info" sub-structure containing information about the
;                    input files and general observation characteristics 
;                    (tag names in parenthesis are specific to alignment
;                    observations):
;                       o NAxis1[Int]         : Number of pixels of the X-axis of the 
;                                               CCD, i.e. number of points of each spec-
;                                               trum = 408 except for window mode.
;                       o NAxis2[Int]         : Number of 5-band spectra recorded throug-
;                                               hout observation. For alignment observa-
;                                               tion this field corresponds to the number
;                                               of pixels of the Y-axis of the CCD = 289.
;                                               For Window modes, it is the vertical width.
;                       o NAxis3[Int]         : Number of bands of the CCD = 5. For align-
;                                               ment observations this is equal to the
;                                               number of complete mapping of the CCD.
;                                               For Window mode this is equal to the number 
;                                               of records.
;                       o Instrument[String]  : Instrument name.
;                       o Orbit[Int]          : Orbit index of the mission.
;                       o Sequence[Int]       : Sequence index of the orbit.
;                       o ObsType[Char]       : Type of observation. Cf SPICAM/V documen
;                                               -tation.
;                       o BeginTime[String]   : Observation first recording time, 
;                                               format=YYYY-MM-DDTHH:MM:SS.mSmSmS.
;                       o EndTime[String]     : Observation last recording time, 
;                                               format=YYYY-MM-DDTHH:MM:SS.mSmSmS
;                       o DCProc [Boolean]    : Dark Charge treatment
;                       o ENProc [Boolean]    : Electronic Noise Treatment
;                       o COSProc [Boolean]   : Cosmic rays pixels Treatment
;                       o SATProc [Boolean]   : Saturated pixels Treatment
;                       o File0C [String]     : 0C file used to build this 1A level
;                       o OrbDCNU [Int]       : Number of the DCNU Model orbit
;                       o Data_status [String]: Status of the Data (F "Final" or "P" Preliminary)
;                       o Geo_status [String] : Status of the Geometry (F "Final" or "P" Preliminary)
;                       o Flag_status [String]: Status of the Flag (F "Final" or "P" Preliminary)
;                       o DC_status [String]  : Status of the Dark Charge correction (F "Final" or "P" Preliminary)
;                     
;                 - "Parameters" sub-structure containing functional parameters 
;                   of the instrument. For 5-bands observations the "All_Ti" and 
;                   "T_..." fields are 1d of length the number of spectra. For 
;                   alignment modes they are 2d with size [nb_PixelY=289,nb_Image]:
;                       o CodeOp[Int]     : Operational code of observation. Cf SPICAM/V docu-
;                                           mentation.
;                       o Binning[Int]    : Number of lines binned together to form a band. In 
;                                           case of progressive binning, the field si set to
;                                           string 'VARIABLE'.
;                       o HT[Int]         : Digital scale of the High Voltage used to amplify 
;                                           signal. Ranging from 0 to 255. See SPICAM/V docu-
;                                           mentation for corresponding voltage.
;                       o Ti[Int]         : Exposure Time of the CCD, in second/100. Usually 
;                                           constant throughout observation. If not the field is 
;                                           set to string 'VARIABLE'.
;                       o X0[Int]         : X-axis first pixel index.
;                       o Y0[Int]         : Y-axis first pixel index. Field is set to 'VARIABLE'
;                                           for alignment observations.
;                       o Slit[boolean]   : 1 if slit is on, 0 else.
;                       o Peltier[boolean]: 1 if Peltier cooler is on, 0 else.
;                       o UVSampling[Int] : Sample rate of the spectrometer.
;                       o IROn[boolean]   : 1 if IR channel is on, 0 else.
;                       o SoirOn[boolean] : 1 if SPICAV SOIR channel is on, 0 else. 255 for 
;                                           SPICAM.
;                       o All_Ti[Int]     : vector of all the time exposure throughout obser-
;                                           vation.
;                       o T_Peltier[Int]  : Temperature vector of Peltier's hot side (all tem-
;                                           peratures ar given in °C).
;                       o T_CCD[Int]      : Temperature vector of the CCD.
;                       o T_NumBoard[Int] : Temperature vector of the digital board.
;                       o T_BTBoard[Int]  : Temperature vector of the low tension board.
;                       o T_Shutter[Int]  : Temperature vector of the solar shutter.
;                       o T_ServBoard[Int]: Temperature vector of the servitudes board.
;                       o T_HVPS[Int]     : Temperature vector of the High Voltage Power
;                                           Supply.
;                       o T_Structure[Int]: Temperature vector of the structure.
;                     
;                 - "GeoInfo" sub-structure containing geometrical parameters 
;                   of the instrument which do not vary during observation:
;                       o Target[String]   : Pointing target of the instrument. Cf SPICAM/V 
;                                            documentation for the star catalog.
;                       o SunLat[Float]    : Sun latitude planet body-fixed frame, at 
;                                            first record (deg).
;                       o SunLong[Float]   : Sun longitude planet body-fixed frame, at
;                                            first record (deg).
;                       o SunDist[ULong]   : Distance between planet and sun, at first
;                                            record (ua).
;                       o SunLS[Float]     : Sun solar longitude in the planet body-fixed
;                                            frame, at first record (deg).
;                       o SunRa[Float]     : Sun Ra, at first record (deg).
;                       o SunDec[Float]    : Sun Dec, at first record (deg).
;                       o SlitCenter[Float]: Position vector of the slit center in SC axes 
;                                            (=offset, deg).
;                       o ShadowCone[Char] : Trajectory of the spacecraft relatively to the 
;                                            planet shadow cone. 
;
;                 - "Geo" sub-structure containing the time, and geometrical
;                   parameters which vary during observation. Parameters are
;                   grouped by the following nested structure. For 5-bands observations
;                   the array is 1d of length the number of spectra. For alignment modes
;                   the array is 2d with size [nb_PixelY=289,nb_Image].
;                   NOTA: Depending on the type and mode of observations, the 
;                   sub-structures vary. Common sub-structures are: {Record,
;                   Spacecraft,Band3,Coordinates,TransMatrix}. All 5-band obser-
;                   vations have the folllowing added sub-structures: {Band1,
;                   Band2,Band4,Band5}. Star occultations have another sub-struc-
;                   ture which is {LineOfSight}. 
;                   Alignment modes have the specific sub-structure {CCDLine}.
;                       o Record: + Number[Int] : observation record number, beginning at 1.
;                                 + Time[String]: Record time, format=YYYY-MM-DDTHH:MM:SS.mSmSmS
;                    
;                       o Spacecraft: + Lat[Float]            : Latitude of the SC (deg).
;                                     + Long[Float]           : Longitude of the SC (deg).
;                                     + Alt[Float]            : Altitude of the SC in (km).
;                                     + SZA[Float]            : Solar zenith angle of the SC (deg).
;                                     + ShadowConeRatio[Float]: Ratio giving position of SC towards 
;                                                               planet shadow cone. If >1 then SC is
;                                                               outside the shadow cone else it's inside.
;
;                       o Band3: + Sza[Float]                : Solar zenith angle of the 3rd band/central 
;                                                              line of the CCD portion (deg).
;                                + DistToPlanetNP[Float]     : Distance to the planet nearest point (km).
;                                + PixelSize[Float]          : Pixel size at planet nearest point (km).
;                                + AngleLOSSun[Float]        : Angle between line of sight and sun (deg).
;                                + PhaseLOSSun[Float]        : Phase between line of sight at planet near-
;                                                              est point and sun(deg).
;                                + SolarIncidence[Float]     : Solar incidence between normal at planet 
;                                                              nearest point and sun (deg)
;                                + SolarLocalTime[Float]     : Solar local time at planet nearest point (deg).
;                                + AngleNormalObserver[Float]: Angle between Normal and Observer (deg).
;                                + DistLOSPlanetCenter[Float]: Distance between line of sight and planet 
;                                                              center (km).
;                      
;                            For non alignment modes only:
;                                + Lat[Float] : Latitude of the 3rd Band  (deg)             
;                                + Long[Float]: Longitude of the 3rd Band (deg)              
;                                + Alt[Float] : Altitude of the 3rd Band (km)               
;                                + Ra[Float]  : Ra of the 3rd Band  (deg)   
;                                + Dec[Float] : Dec of the 3rd Band (deg)
;
;                       o Band1: + Lat[Float] : Latitude of the 1st Band (deg)              
;                                + Long[Float]: Longitude of the 1st Band (deg)              
;                                + Alt[Float] : Altitude of the 1st Band (km)               
;                                + Ra[Float]  : Ra of the 1st Band (deg)     
;                                + Dec[Float] : Dec of the 1st Band (deg)
;
;                       o Band2,Band4 and Band5 have the same corresponding fields.
;
;                       o CCDLine has the same corresponding fields for each line 
;                         of the CCD. For alignment observations this structure replaces
;                         the missing Band3 fields.
;
;                       o LOSE: Same fields as Band3 common fields, but associated to 
;                               the line of sight emanating from the UV CCD center, but 
;                               defined by S/C position and Star direction.
; 
;                       o Coordinates: + Xsc_X[Float]: X coordinate of the SC X-axis in MARSIAU/IAU_VENUS.
;                                      + Xsc_Y[Float]: Y coordinate of the SC X-axis in MARSIAU/IAU_VENUS.
;                                      + Xsc_Z[Float]: Z coordinate of the SC X-axis in MARSIAU/IAU_VENUS.
;                                      + Ysc_X[Float]: X coordinate of the SC Y-axis in MARSIAU/IAU_VENUS.
;                                      + Ysc_Y[Float]: Y coordinate of the SC Y-axis in MARSIAU/IAU_VENUS.
;                                      + Ysc_Z[Float]: Z coordinate of the SC Y-axis in MARSIAU/IAU_VENUS.
;                                      + Zsc_X[Float]: X coordinate of the SC Z-axis in MARSIAU/IAU_VENUS.
;                                      + Zsc_Y[Float]: Y coordinate of the SC Z-axis in MARSIAU/IAU_VENUS.
;                                      + Zsc_Z[Float]: Z coordinate of the SC Z-axis in MARSIAU/IAU_VENUS.
;                                      + Angle_Xsc_LocalVert[Float]: Angle between Xsc and local verti-
;                                                                    cal at planet nearest point (deg).
;                                      + Angle_Xsc_Proj[Float]     : Angle between Xsc and the projec-
;                                                                    tion plan (PP) U-axis (deg).
;                                      + P1_u[Float]: U coordinate of the horizon point in the PP.
;                                      + P1_v[Float]: V coordinate of the horizon point in the PP.
;                                      + P2_u[Float]: U coordinate of the horizon point in the PP.
;                                      + P2_v[Float]: V coordinate of the horizon point in the PP.
;                                      + P3_u[Float]: U coordinate of the horizon point in the PP.
;                                      + P3_v[Float]: V coordinate of the horizon point in the PP.
;                                      + P4_u[Float]: U coordinate of the horizon point in the PP.
;                                      + P4_v[Float]: V coordinate of the horizon point in the PP.
;                                      + P5_u[Float]: U coordinate of the horizon point in the PP.
;                                      + P5_v[Float]: V coordinate of the horizon point in the PP.
;
;                       o TransMatrix: + X_Ra [Float]: Ra of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
;                                                      formation matrix X vector (deg).
;                                      + X_Dec[Float]: Dec of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
;                                                      formation matrix X vector (deg).
;                                      + Y_Ra [Float]: Ra of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
;                                                      formation matrix Y vector (deg).
;                                      + Y_Dec[Float]: Dec of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
;                                                      formation matrix Y vector (deg).
;                                      + Z_Ra [Float]: Ra of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
;                                                      formation matrix Z vector (deg).
;                                      + Z_Dec[Float]: Dec of the MARSIAU/IAU_VENUS to EMEJ2000 trans-
;                                                      formation matrix Z vector (deg).
;
;                 - "GeoUnits" sub-structure, with the same architecture as "Geo". For
;                   each field, the corresponding unit is given as a string.
;
;        In case of error during the reading process, the program will print out a message 
;        error and return a boolean 0 (False).
;                                  
; OPT. OUTPUTS:       
;      None.   
;
; KEYWORDS: 
;
;      SILENT   = If the /SILENT keyword is not set in the call, the function will
;                 print a message to confirm the success of the reading process.
;
;
; EXAMPLES: Read a SPICAM 1A file and plot central band:
;           Obs = SPICA_Read_1A(fname_Data1A)
;           TVIMAGE, Obs.CleanData[*,*,2]
;
;           Show the 3rd band SZA of the first 100 records:
;           Print, Obs.Geo.Band3[0:99].SZA
;
;           If Obs was an Alignment observation and you wish to know the SZA of the 
;           7th image that would be:
;           Print, Obs.Geo.Band3[*,6].SZA
;
; CALLS:
;       The function SPICA_Read_1A calls the following procedures of the IDL Astronomy 
;       Library available at http://idlastro.gsfc.nasa.gov/fitsio.html:
;       FITS_OPEN(), FXADDPAR ,FXPAR(), FXPARPOS(), FXMOVE(), FXPOSIT(), GETTOK(), 
;       MRD_HREAD, MRDFITS(), MRD_SKIP, MRD_STRUCT(), SXDELPAR(), SXPAR(), VALID_NUM()
;       
;
; COMMON: 
;       None.
;
; RESTRICTIONS: 
;       Use only to read SPICAM/V Data 1A file with name syntax
;       "SPIM_1AU_xxxxxAyy_t_01.FITS" or "SPIV_1AU_xxxxAyy_t_01.FITS"
;       
; SVN revision information:
;       $Rev: 940 $
;

;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
FUNCTION SPICA_Read_1A,fname_Data1A,SILENT=Silent


  ;--------------------------------------------------------------
  ;---------------------- PRELIMINARIES -------------------------

  ;--- Verifying argument and existence of file ---
  IF N_Params() NE 1 THEN BEGIN
    Message,'Incorrect number of arguments!'                                      $
            + String(13B)                                                         $
            + 'Syntax: Output=SPICA_Read_1A(Filename,/SILENT).'                   $
            + String(13B)                                                         $
            + 'File reading aborted',/CONTINUE
    Return,0b
  ENDIF
    
  IF (Size(fname_Data1A))[1] NE 7 THEN BEGIN    
    Message,'Not consistent filename argument!'                                   $
            + String(13B)                                                         $
            + 'Please give a data 1A filename of string type.'                    $
            + String(13B)                                                         $
            + 'File reading aborted.',/CONTINUE
    Return,0b
  ENDIF
    
  IF File_Test(fname_Data1A) EQ 0 THEN BEGIN
    Message,String(fname_Data1A,FORMAT='("The file ",A," could not be found!")')  $
            + String(13B)                                                         $
            + 'Please check the file existence and filename.'                     $
            + String(13B)                                                         $
            + 'File reading aborted.',/CONTINUE
    Return,0b
  ENDIF
  ;----------------------------------
  
  
  ;--- Scanning file contents ---
  Fits_Open,fname_Data1A,fcb_Data1A,/NO_ABORT
  
  ;--- Verifying FITS standards ---
  If N_Elements(fcb_Data1A) EQ 0 THEN BEGIN
    Message,'Contents are not coherent for file: '+File_Basename(fname_Data1A)   $
            + String(13B)                                                        $
            + 'Please make sure the file is a SPICAM/V 1A FITS file.'            $
            + String(13B)                                                        $
            + 'File reading aborted.',/CONTINUE
    Return,0b
  ENDIF
  
  ;--- Checking essential extensions ---
  bool_Param  = Max(StrMatch(fcb_Data1A.ExtName,'Functional_Parameters'))
  bool_Flag   = Max(StrMatch(fcb_Data1A.ExtName,'Flag'))
  bool_Err    = Max(StrMatch(fcb_Data1A.ExtName,'ErrData'))
  bool_GeoRec = Max(StrMatch(fcb_Data1A.ExtName,'Geo_Record'))
  bool_GeoSC  = Max(StrMatch(fcb_Data1A.ExtName,'Geo_Spacecraft'))
  bool_GeoB3  = Max(StrMatch(fcb_Data1A.ExtName,'Geo_Band3'))
  bool_GeoCo  = Max(StrMatch(fcb_Data1A.ExtName,'Geo_Coordinates'))
  bool_GeoTM  = Max(StrMatch(fcb_Data1A.ExtName,'Geo_TransMatrix'))
  
  bool_Common =  bool_Param  $
             AND bool_Flag   $ 
             AND bool_Err    $ 
             AND bool_GeoRec $
             AND bool_GeoSC  $
             AND bool_GeoB3  $
             AND bool_GeoCo  $
             AND bool_GeoTM
  
  IF bool_Common NE 1 THEN BEGIN
    Message,'Missing essential extension for file: '+File_Basename(fname_Data1A) $
            + String(13B)                                                        $
            + 'Please make sure the file is a SPICAM/V 1A FITS file.'            $
            + String(13B)                                                        $
            + 'File reading aborted.',/CONTINUE
    Return,0b
  ENDIF
  ;-------------------------------------
  
  
  ;--- Checking mode specific extensions ---
  bool_GeoB1  = Max(StrMatch(fcb_Data1A.ExtName,'Geo_Band1'))
  bool_GeoB2  = Max(StrMatch(fcb_Data1A.ExtName,'Geo_Band2'))
  bool_GeoB4  = Max(StrMatch(fcb_Data1A.ExtName,'Geo_Band4'))
  bool_GeoB5  = Max(StrMatch(fcb_Data1A.ExtName,'Geo_Band5'))
  bool_GeoCCD = Max(StrMatch(fcb_Data1A.ExtName,'Geo_CCDLine'))
  bool_GeoLOS = Max(StrMatch(fcb_Data1A.ExtName,'Geo_LOSE'))
  
  ;---------------------- END OF PRELIMINAIRES -----------------
  ;-------------------------------------------------------------





  ;--------------------------------------------------------------------
  ;----------------------- READING PRIMARY DATA -----------------------

  ;--- Reading primary data and header ---
  arr3_Data = MRdFits(fname_Data1A,0,fitshdr_Primary,/SILENT)
  arr3_Data = Float(arr3_Data)
  
  ;--- Retrieving general info variables ---
  nb_Axis1  = FXPar(fitshdr_Primary,'NAXIS1')
  nb_Axis2  = FXPar(fitshdr_Primary,'NAXIS2')
  nb_Axis3  = FXPar(fitshdr_Primary,'NAXIS3')
  str_Instr = FXPar(fitshdr_Primary,'INSTRU')
  Orbit     = FXPar(fitshdr_Primary,'ORBIT')
  SeqNumber = FXPar(fitshdr_Primary,'SEQ_NB')
  ObsType   = StrTrim(FXPar(fitshdr_Primary,'OBSTYPE'))
  IF StrMatch(ObsType,'*1') THEN ObsType='T'
  IF StrMatch(ObsType,'*0') THEN ObsType='F'
  BeginTime = FXPar(fitshdr_Primary,'BEGINS')
  EndTime   = FXPar(fitshdr_Primary,'ENDS')
  dcp = fxpar(fitshdr_Primary,'DCProc')
  enp = fxpar(fitshdr_Primary,'ENProc')
  cosp = fxpar(fitshdr_Primary,'COSProc') 
  satp = fxpar(fitshdr_Primary,'SATProc')
  file0C = fxpar(fitshdr_Primary,'0C')
  OrbDCNU = fxpar(fitshdr_Primary,'ORBDCNU')
  DATA_STATUS = StrTrim(fxpar(fitshdr_Primary,'DATA_SS'))
  GEO_STATUS = StrTrim(fxpar(fitshdr_Primary,'GEO_SS'))
  FLAG_STATUS = StrTrim(fxpar(fitshdr_Primary,'FLAG_SS'))
  DC_STATUS = StrTrim(fxpar(fitshdr_Primary,'DC_SS'))
     
  ;------------------ END OF READING PRIMARY DATA ---------------------
  ;--------------------------------------------------------------------
    
    
  
  
  
  ;--------------------------------------------------------------------
  ;------------- READING FUNCTIONAL PARAMETERS EXTENSION --------------

  ;--- Reading functional parameters extension and header ---
  struc_VarParameters = MRdFits(fname_Data1A,'Functional_Parameters',fitshdr_FuncParams,/SILENT)
  
  ;--- Retrieving functional parameters variables ---                 
  CodeOp  = FXPar(fitshdr_FuncParams,'CODEOP')
  Binning = FXPar(fitshdr_FuncParams,'BINNING')
  HT      = FXPar(fitshdr_FuncParams,'HT')
  Ti      = FXPar(fitshdr_FuncParams,'TI')
  X0      = FXPar(fitshdr_FuncParams,'X0')
  Y0      = FXPar(fitshdr_FuncParams,'Y0')
  Slit    = FXPar(fitshdr_FuncParams,'SLIT')
  Peltier = FXPar(fitshdr_FuncParams,'PELTIER')
  UVSampl = FXPar(fitshdr_FuncParams,'UVSAMPL')
  IROn    = FXPar(fitshdr_FuncParams,'IR_ON')
  SoirOn  = FXPar(fitshdr_FuncParams,'SOIR_ON')
     
  ;---------- END OF READING FUNCTIONAL PARAMETERS EXTENSION ----------
  ;--------------------------------------------------------------------
  
  
  
  
  
  ;--------------------------------------------------------------------
  ;--------------------- READING FLAG EXTENSION -----------------------

  ;--- Reading the flag structure ---
  arr3_Flag = MRdFits(fname_Data1A,'Flag',/SILENT)
    
  ;------------------ END OF READING FLAG EXTENSION -------------------
  ;--------------------------------------------------------------------  
    
    
    
    
  ;--------------------------------------------------------------------
  ;--------------------- READING ERRORDATA EXTENSION -----------------------

  ;--- Reading the flag structure ---
  arr3_Err = MRdFits(fname_Data1A,'ErrData',/SILENT)
    
  ;------------------ END OF READING ERRORDATA EXTENSION -------------------
  ;--------------------------------------------------------------------  

  
  
  
  ;--------------------------------------------------------------------
  ;--------------------- READING GEO EXTENSIONS -----------------------
 
  ;--- Reading common geo extensions and first extension header---
  struc_GeoRecord      = MRdFits(fname_Data1A,'Geo_Record',fitshdr_GeoRecord,/SILENT)
  struc_GeoSpacecraft  = MRdFits(fname_Data1A,'Geo_Spacecraft',fitshdr_GeoSpacecraft,/SILENT)
  struc_GeoBand3       = MRdFits(fname_Data1A,'Geo_Band3',fitshdr_GeoBand3,/SILENT)
  struc_GeoCoordinates = MRdFits(fname_Data1A,'Geo_Coordinates',fitshdr_GeoCoordinates,/SILENT)
  struc_GeoTransMatrix = MRdFits(fname_Data1A,'Geo_TransMatrix',fitshdr_GeoTransMatrix,/SILENT)
  
  
  ;--- Checking specific geo extensions ---
  IF CodeOp EQ 100                 $
  THEN bool_Specific = bool_GeoCCD $
  ELSE bool_Specific = bool_GeoB1  $
                   AND bool_GeoB2  $
                   AND bool_GeoB4  $
                   AND bool_GeoB5
  
  IF bool_Specific NE 1 THEN BEGIN
    Message,'Missing specific extension for file '+File_Basename(fname_Data1A)  $
            + String(13B)                                                       $
            + 'Please make sure the file is a SPICAM/V 1A FITS file.'           $
            + String(13B)                                                       $
            + 'File reading aborted.',/CONTINUE
    Return,0b
  ENDIF
  ;----------------------------------------
  
  
  ;--- Reading specific geo extensions ---
  IF bool_GeoCCD THEN struc_GeoCCDLine = MRdFits(fname_Data1A,'Geo_CCDLine',fitshdr_GeoCCDLine,/SILENT)  
  IF bool_GeoB1  THEN struc_GeoBand1   = MRdFits(fname_Data1A,'Geo_Band1',fitshdr_GeoBand1,/SILENT)
  IF bool_GeoB2  THEN struc_GeoBand2   = MRdFits(fname_Data1A,'Geo_Band2',fitshdr_GeoBand2,/SILENT)
  IF bool_GeoB4  THEN struc_GeoBand4   = MRdFits(fname_Data1A,'Geo_Band4',fitshdr_GeoBand4,/SILENT)
  IF bool_GeoB5  THEN struc_GeoBand5   = MRdFits(fname_Data1A,'Geo_Band5',fitshdr_GeoBand5,/SILENT)
  IF bool_GeoLOS THEN struc_GeoLOS     = MRdFits(fname_Data1A,'Geo_LOSE',fitshdr_GeoLOSE,/SILENT)
  
  Fits_Close, fcb_Data1A
  
  ;------------------ END OF READING GEO EXTENSIONS -------------------
  ;--------------------------------------------------------------------
  
  
  
  
  
  ;--------------------------------------------------------------------
  ;------------------- BUILDING OUTPUT SUB-STRUCTURES -----------------
  
  ;--- Building the Info and Parameters structures ---
  struc_Info = {NAxis1:nb_Axis1,              $
                NAxis2:nb_Axis2,              $
                NAxis3:nb_Axis3,              $
                Instrument:str_Instr,         $
                Orbit:Orbit,                  $
                Sequence:SeqNumber,           $
                ObsType:ObsType,              $
                BeginTime:BeginTime,          $
                EndTime:EndTime,              $
                DCProc:dcp,                   $
                orbDCNU:ORBDCNU,              $
                ENProc:enp,                   $
                COSProc:cosp,                 $
                SATProc:satp,                 $
                Data_status:DATA_STATUS,      $
                Geo_status:GEO_STATUS,        $
                Flag_status:FLAG_STATUS,      $
                DC_status:DC_STATUS}
  
  struc_FixParameters = {CodeOp:CodeOp,       $
                         Binning:Binning,     $
                         HT:HT,               $
                         Ti:Ti,               $
                         X0:X0,               $
                         Y0:Y0,               $
                         Slit:Slit,           $
                         Peltier:Peltier,     $
                         UVSampling:UVSampl,  $
                         IROn:IROn,           $
                         SoirOn:SoirOn}
  
   CASE CodeOp OF 
                                                      
    100:  struc_VarParameters = {All_Ti:Reform(struc_VarParameters.All_Ti,nb_Axis2,nb_Axis3),            $
                                 T_Peltier:Reform(struc_VarParameters.T_Peltier,nb_Axis2,nb_Axis3),      $
                                 T_CCD:Reform(struc_VarParameters.T_CCD,nb_Axis2,nb_Axis3),              $
                                 T_NumBoard:Reform(struc_VarParameters.T_NumBoard,nb_Axis2,nb_Axis3),    $ 
                                 T_BTBoard:Reform(struc_VarParameters.T_BTBoard,nb_Axis2,nb_Axis3),      $
                                 T_Shutter:Reform(struc_VarParameters.T_Shutter,nb_Axis2,nb_Axis3),      $
                                 T_ServBoard:Reform(struc_VarParameters.T_ServBoard,nb_Axis2,nb_Axis3),  $
                                 T_HVPS:Reform(struc_VarParameters.T_HVPS,nb_Axis2,nb_Axis3),            $
                                 T_Structure:Reform(struc_VarParameters.T_Structure,nb_Axis2,nb_Axis3)}
    
    ELSE: struc_VarParameters = {All_Ti:struc_VarParameters.All_Ti,            $
                                 T_Peltier:struc_VarParameters.T_Peltier,      $
                                 T_CCD:struc_VarParameters.T_CCD,              $
                                 T_NumBoard:struc_VarParameters.T_NumBoard,    $ 
                                 T_BTBoard:struc_VarParameters.T_BTBoard,      $
                                 T_Shutter:struc_VarParameters.T_Shutter,      $
                                 T_ServBoard:struc_VarParameters.T_ServBoard,  $
                                 T_HVPS:struc_VarParameters.T_HVPS,            $
                                 T_Structure:struc_VarParameters.T_Structure}
  
  ENDCASE
                                   
  struc_Parameters = Create_Struct(struc_FixParameters,struc_VarParameters)
  ;---------------------------------------------------
  
  
  ;--- Building the GeoHeader structure ---
  str_Target = StrTrim(FXPar(fitshdr_GeoRecord,'TARGET'),2)
  Lat        = FXPar(fitshdr_GeoRecord,'SUNLAT')
  Long       = FXPar(fitshdr_GeoRecord,'SUNLONG')
  Dist       = FXPar(fitshdr_GeoRecord,'SUNDIST')
  LS         = FXPar(fitshdr_GeoRecord,'SUNLS')
  Dec        = FXPar(fitshdr_GeoRecord,'SUNDEC')
  Ra         = FXPar(fitshdr_GeoRecord,'SUNRA')
  str_Slit   = FXPar(fitshdr_GeoRecord,'SLIT_C')
  Cone       = StrTrim(FXPar(fitshdr_GeoRecord,'CONE'),2)
      
  vec_SlitCenter = Float(StrSplit(str_Slit,/EXTRACT)) 
  
  struc_GeoInfo = {Target:str_Target,         $
                   SunLat:Lat,                $
                   SunLong:Long,              $
                   SunDist:Dist,              $
                   SunLS:LS,                  $
                   SunRa:Ra,                  $
                   SunDec:Dec,                $
                   SlitCenter:vec_SlitCenter, $
                   ShadowCone:Cone}
  ;-----------------------------------------
                   
                   
  ;--- Building the GeoData structure ---                 
  CASE CodeOp OF 
                                                      
    100: struc_GeoData = {Record     :Reform(struc_GeoRecord,nb_Axis2,nb_Axis3),       $
                          Spacecraft :Reform(struc_GeoSpacecraft,nb_Axis2,nb_Axis3),   $
                          CCDLine    :Reform(struc_GeoCCDLine,nb_Axis2,nb_Axis3),      $ 
                          Band3      :Reform(struc_GeoBand3,nb_Axis2,nb_Axis3),        $
                          Coordinates:Reform(struc_GeoCoordinates,nb_Axis2,nb_Axis3),  $
                          TransMatrix:Reform(struc_GeoTransMatrix,nb_Axis2,nb_Axis3)}
                        
    ELSE: struc_GeoData = {Record     :struc_GeoRecord,             $
                           Spacecraft :struc_GeoSpacecraft,         $
                           Band1      :struc_GeoBand1,              $
                           Band2      :struc_GeoBand2,              $
                           Band3      :struc_GeoBand3,              $
                           Band4      :struc_GeoBand4,              $
                           Band5      :struc_GeoBand5,              $   
                           Coordinates:struc_GeoCoordinates,        $
                           TransMatrix:struc_GeoTransMatrix}
  ENDCASE
     
  IF bool_GeoLOS                                                                $
  THEN struc_GeoData = Create_Struct(struc_GeoData,'LOSE',struc_GeoLOS)
  ;-------------------------------------- 
  
  
  ;--- Building the GeoUnits structure --- 
  vec_Sub = Tag_Names(struc_GeoData)
  com = 'struc_GeoUnits = Create_Struct(vec_Sub'
  
  FOR i_Sub=0,N_Elements(vec_Sub)-1 DO BEGIN
    
    void = Execute('vec_SubTagNames = Tag_Names(struc_GeoData.' + vec_Sub[i_Sub] + ')' )  
    void = Execute('vec_Fields = FXPAR(fitshdr_Geo' + vec_Sub[i_Sub] + ",'TTYPE*',COMMENT=vec_Units)")
    
    ind_NoUnits = Where(vec_Units EQ '',nb_NoUnits)
    IF nb_NoUnits NE 0                 $
    THEN vec_Units(ind_NoUnits) = 'N/A'
    
    subcom = '{' + vec_SubTagNames[0] + ":'" + vec_Units[0] + "'"
    FOR i_SubTag=1,N_Elements(vec_SubTagNames)-1                          $
    DO subcom = subcom + ','                                              $
              + vec_SubTagNames[i_SubTag] +":'"+vec_Units[i_SubTag] + "'"
    
    com = com + ',' + subcom + '}'
    
  ENDFOR
  
  com = com + ')'
  
  void = Execute(com)
  ;-------------------------------------- 


  ;---------------- END OF BUILDING OUTPUT SUB-STRUCTURES ---------------
  ;----------------------------------------------------------------------


  ;--- Building the final output structure ---
  struc_1AFile = {CleanData :arr3_Data,          $
                  Info      :struc_Info,         $
                  Parameters:struc_Parameters,   $
                  Flag      :arr3_Flag,          $
                  ErrData   :arr3_Err,           $
                  GeoInfo   :struc_GeoInfo,      $
                  Geo       :struc_GeoData,      $
                  GeoUnits  :struc_GeoUnits}
  
  ;--- Information for user ---
  IF ~Keyword_Set(Silent)                                                         $
  THEN Print,'The file ' + File_Basename(fname_Data1A) + ' was read succesfully!'
  
  ;--- Ejecting ---
  Return,struc_1AFile
  
END
;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<